@@ -21,7 +21,7 @@
  * Copyright (C) 2006 Miguel Covarrubias <mlcovarrubias@gmail.com>
  * Copyright (C) 2006 Michael Studman <codehaus@michaelstudman.com>
  * Copyright (C) 2008 Joseph LaFata <joe@quibb.org>
- * 
+ *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
  * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
@@ -36,6 +36,8 @@
  ***** END LICENSE BLOCK *****/
 package org.jruby;
 
+import java.util.Collections;
+import java.util.Map;
 import jnr.posix.POSIX;
 import org.jcodings.Encoding;
 import org.jruby.anno.JRubyMethod;
@@ -63,11 +65,8 @@ import org.jruby.util.io.Stream;
 
 import static org.jruby.internal.runtime.GlobalVariable.Scope.*;
 
-import java.util.HashMap;
-import java.util.Map;
-
 /** This class initializes global variables and constants.
- * 
+ *
  * @author jpetersen
  */
 public class RubyGlobal {
@@ -92,7 +91,7 @@ public class RubyGlobal {
         GlobalVariables globals = runtime.getGlobalVariables();
 
         runtime.defineGlobalConstant("TOPLEVEL_BINDING", runtime.newBinding());
-        
+
         runtime.defineGlobalConstant("TRUE", runtime.getTrue());
         runtime.defineGlobalConstant("FALSE", runtime.getFalse());
         runtime.defineGlobalConstant("NIL", runtime.getNil());
@@ -139,7 +138,7 @@ public class RubyGlobal {
 
         runtime.defineGlobalConstant("RELEASE_DATE", release);
         runtime.defineGlobalConstant("PLATFORM", platform);
-        
+
         IRubyObject jrubyVersion = runtime.newString(Constants.VERSION).freeze(context);
         IRubyObject jrubyRevision = runtime.newString(Constants.REVISION).freeze(context);
         runtime.defineGlobalConstant("JRUBY_VERSION", jrubyVersion);
@@ -155,13 +154,13 @@ public class RubyGlobal {
             // needs to be a fixnum, but our revision is a sha1 hash from git
             runtime.defineGlobalConstant("RUBY_REVISION", runtime.newFixnum(Constants.RUBY1_9_REVISION));
         }
-        
+
         if (runtime.is1_9()) {
             RubyInstanceConfig.Verbosity verbosity = runtime.getInstanceConfig().getVerbosity();
             runtime.defineVariable(new WarningGlobalVariable(runtime, "$-W", verbosity), GLOBAL);
         }
 
-        final GlobalVariable kcodeGV; 
+        final GlobalVariable kcodeGV;
         if (runtime.is1_9()) {
             kcodeGV = new NonEffectiveGlobalVariable(runtime, "$KCODE", runtime.getNil());
         } else {
@@ -190,7 +189,7 @@ public class RubyGlobal {
         } else {
             runtime.defineVariable(new GlobalVariable(runtime, "$;", RubyRegexp.newRegexp(runtime, runtime.getInstanceConfig().getInputFieldSeparator(), new RegexpOptions())), GLOBAL);
         }
-        
+
         RubyInstanceConfig.Verbosity verbose = runtime.getInstanceConfig().getVerbosity();
         IRubyObject verboseValue = null;
         if (verbose == RubyInstanceConfig.Verbosity.NIL) {
@@ -203,7 +202,7 @@ public class RubyGlobal {
         runtime.defineVariable(new VerboseGlobalVariable(runtime, "$VERBOSE", verboseValue), GLOBAL);
         runtime.defineVariable(new VerboseGlobalVariable(runtime, "$-v", verboseValue), GLOBAL);
         runtime.defineVariable(new VerboseGlobalVariable(runtime, "$-w", verboseValue), GLOBAL);
-        
+
         IRubyObject debug = runtime.newBoolean(runtime.getInstanceConfig().isDebug());
         runtime.defineVariable(new DebugGlobalVariable(runtime, "$DEBUG", debug), GLOBAL);
         runtime.defineVariable(new DebugGlobalVariable(runtime, "$-d", debug), GLOBAL);
@@ -235,26 +234,26 @@ public class RubyGlobal {
         runtime.defineVariable(new LoadPath(runtime, "$:"), GLOBAL);
         runtime.defineVariable(new LoadPath(runtime, "$-I"), GLOBAL);
         runtime.defineVariable(new LoadPath(runtime, "$LOAD_PATH"), GLOBAL);
-        
+
         runtime.defineVariable(new MatchMatchGlobalVariable(runtime, "$&"), FRAME);
         runtime.defineVariable(new PreMatchGlobalVariable(runtime, "$`"), FRAME);
         runtime.defineVariable(new PostMatchGlobalVariable(runtime, "$'"), FRAME);
         runtime.defineVariable(new LastMatchGlobalVariable(runtime, "$+"), FRAME);
         runtime.defineVariable(new BackRefGlobalVariable(runtime, "$~"), FRAME);
 
-        // On platforms without a c-library accessable through JNA, getpid will return hashCode 
+        // On platforms without a c-library accessable through JNA, getpid will return hashCode
         // as $$ used to. Using $$ to kill processes could take down many runtimes, but by basing
         // $$ on getpid() where available, we have the same semantics as MRI.
         globals.defineReadonly("$$", new PidAccessor(runtime), GLOBAL);
 
         // after defn of $stderr as the call may produce warnings
         defineGlobalEnvConstants(runtime);
-        
-        // Fixme: Do we need the check or does Main.java not call this...they should consolidate 
+
+        // Fixme: Do we need the check or does Main.java not call this...they should consolidate
         if (globals.get("$*").isNil()) {
             globals.defineReadonly("$*", new ValueAccessor(runtime.newArray()), GLOBAL);
         }
-        
+
         globals.defineReadonly("$-p",
                 new ValueAccessor(runtime.newBoolean(runtime.getInstanceConfig().isAssumePrinting())),
                 GLOBAL);
@@ -296,31 +295,27 @@ public class RubyGlobal {
         globals.alias("$LAST_PAREN_MATCH", "$+");
     }
 
+    @SuppressWarnings("unchecked")
     private static void defineGlobalEnvConstants(Ruby runtime) {
-    	Map environmentVariableMap = null;
-    	OSEnvironment environment = new OSEnvironment();
-        environmentVariableMap = environment.getEnvironmentVariableMap(runtime);
-		
-    	if (environmentVariableMap == null) {
-            // if the environment variables can't be obtained, define an empty ENV
-    		environmentVariableMap = new HashMap();
-    	}
-
-    	CaseInsensitiveStringOnlyRubyHash env = new CaseInsensitiveStringOnlyRubyHash(runtime,
-                                                       environmentVariableMap, 
-                                                       runtime.getNil(),
-                                                       runtime.getInstanceConfig().isNativeEnabled() && 
-                                                           runtime.getInstanceConfig().isUpdateNativeENVEnabled() );
+    	Map<RubyString, RubyString> environmentVariableMap = OSEnvironment.environmentVariableMap(runtime);
+    	RubyHash env = new CaseInsensitiveStringOnlyRubyHash(
+            runtime, environmentVariableMap, runtime.getNil(),
+            runtime.getInstanceConfig().isNativeEnabled() && runtime.getInstanceConfig().isUpdateNativeENVEnabled()
+        );
         env.getSingletonClass().defineAnnotatedMethods(CaseInsensitiveStringOnlyRubyHash.class);
+        if ( ! runtime.is2_0() ) { // MRI 1.9.3 does not have ENV#to_h
+            env.getSingletonClass().undefineMethod("to_h");
+        }
         runtime.defineGlobalConstant("ENV", env);
         runtime.setENV(env);
 
         // Define System.getProperties() in ENV_JAVA
-        Map systemProps = environment.getSystemPropertiesMap(runtime);
-        RubyHash systemPropsHash = new ReadOnlySystemPropertiesHash(
-                runtime, systemProps, runtime.getNil());
-        systemPropsHash.setFrozen(true);
-        runtime.defineGlobalConstant("ENV_JAVA", systemPropsHash);
+        Map<RubyString, RubyString> systemPropertiesMap = OSEnvironment.systemPropertiesMap(runtime);
+        RubyHash envJava = new ReadOnlySystemPropertiesHash(
+                runtime, systemPropertiesMap, runtime.getNil()
+        );
+        envJava.setFrozen(true);
+        runtime.defineGlobalConstant("ENV_JAVA", envJava);
     }
 
     /**
@@ -344,12 +339,22 @@ public class RubyGlobal {
             return case_aware_op_aset(context, key, value, false);
         }
 
-        @JRubyMethod
+        @JRubyMethod(name = "to_s")
+        public IRubyObject to_s(ThreadContext context) {
+            return context.runtime.newString("ENV");
+        }
+
         @Override
-        public IRubyObject to_s(){
+        public IRubyObject to_s() {
             return getRuntime().newString("ENV");
         }
 
+        @JRubyMethod
+        public RubyHash to_h(){
+          return to_hash();
+        }
+
+
     }
 
     /**
@@ -362,7 +367,7 @@ public class RubyGlobal {
         // class and not in the caseinsensitive hash.  In order to not refactor
         // both of these maps we will pass in a flag to specify whether we want
         // the op_aset to also update the real ENV map via setenv/unsetenv.
-        private boolean updateRealENV;
+        private final boolean updateRealENV;
 
         public StringOnlyRubyHash(Ruby runtime, Map valueMap, IRubyObject defaultValue, boolean updateRealENV) {
             super(runtime, valueMap, defaultValue);
@@ -404,7 +409,8 @@ public class RubyGlobal {
             return super.op_aref(context, key);
         }
 
-        protected IRubyObject case_aware_op_aset(ThreadContext context, IRubyObject key, IRubyObject value, boolean caseSensitive) {
+        protected IRubyObject case_aware_op_aset(ThreadContext context,
+            IRubyObject key, final IRubyObject value, boolean caseSensitive) {
             if (!key.respondsTo("to_str")) {
                 throw getRuntime().newTypeError("can't convert " + key.getMetaClass() + " into String");
             }
@@ -412,7 +418,7 @@ public class RubyGlobal {
                 throw getRuntime().newTypeError("can't convert " + value.getMetaClass() + " into String");
             }
 
-            if (! caseSensitive) {
+            if (!caseSensitive) {
                 key = getCorrectKey(key, context);
             }
 
@@ -421,14 +427,14 @@ public class RubyGlobal {
             }
 
             IRubyObject keyAsStr = normalizeEnvString(Helpers.invoke(context, key, "to_str"));
-            IRubyObject valueAsStr = value.isNil() ? getRuntime().getNil() :
+            IRubyObject valueAsStr = value.isNil() ? context.nil :
                     normalizeEnvString(Helpers.invoke(context, value, "to_str"));
 
             if (updateRealENV) {
-                POSIX posix = getRuntime().getPosix();
+                POSIX posix = context.runtime.getPosix();
                 String keyAsJava = keyAsStr.asJavaString();
                 // libc (un)setenv is not reentrant, so we need to synchronize across the entire JVM (JRUBY-5933)
-                if (valueAsStr == getRuntime().getNil()) {
+                if (valueAsStr == context.nil) {
                     synchronized (Object.class) { posix.unsetenv(keyAsJava); }
                 } else {
                     synchronized (Object.class) { posix.setenv(keyAsJava, valueAsStr.asJavaString(), 1); }
@@ -464,9 +470,8 @@ public class RubyGlobal {
                 RubyString newStr = getRuntime().newString(new ByteList(str.toString().getBytes(), enc));
                 newStr.setFrozen(true);
                 return newStr;
-            } else {
-                return str;
             }
+            return str;
         }
     }
 
@@ -508,13 +513,13 @@ public class RubyGlobal {
         public LastExitStatusVariable(Ruby runtime, String name) {
             super(runtime, name, runtime.getNil());
         }
-        
+
         @Override
         public IRubyObject get() {
             IRubyObject lastExitStatus = runtime.getCurrentContext().getLastExitStatus();
             return lastExitStatus == null ? runtime.getNil() : lastExitStatus;
         }
-        
+
         @Override
         public IRubyObject set(IRubyObject lastExitStatus) {
             throw runtime.newNameError("$? is a read-only variable", "$?");
@@ -525,7 +530,7 @@ public class RubyGlobal {
         public MatchMatchGlobalVariable(Ruby runtime, String name) {
             super(runtime, name, runtime.getNil());
         }
-        
+
         @Override
         public IRubyObject get() {
             return RubyRegexp.last_match(runtime.getCurrentContext().getBackRef());
@@ -536,7 +541,7 @@ public class RubyGlobal {
         public PreMatchGlobalVariable(Ruby runtime, String name) {
             super(runtime, name, runtime.getNil());
         }
-        
+
         @Override
         public IRubyObject get() {
             return RubyRegexp.match_pre(runtime.getCurrentContext().getBackRef());
@@ -547,7 +552,7 @@ public class RubyGlobal {
         public PostMatchGlobalVariable(Ruby runtime, String name) {
             super(runtime, name, runtime.getNil());
         }
-        
+
         @Override
         public IRubyObject get() {
             return RubyRegexp.match_post(runtime.getCurrentContext().getBackRef());
@@ -558,7 +563,7 @@ public class RubyGlobal {
         public LastMatchGlobalVariable(Ruby runtime, String name) {
             super(runtime, name, runtime.getNil());
         }
-        
+
         @Override
         public IRubyObject get() {
             return RubyRegexp.match_last(runtime.getCurrentContext().getBackRef());
@@ -569,7 +574,7 @@ public class RubyGlobal {
         public BackRefGlobalVariable(Ruby runtime, String name) {
             super(runtime, name, runtime.getNil());
         }
-        
+
         @Override
         public IRubyObject get() {
             return Helpers.getBackref(runtime, runtime.getCurrentContext());
@@ -616,7 +621,7 @@ public class RubyGlobal {
                     !(JavaUtil.isJavaObject(value) && JavaUtil.unwrapJavaObject(value) instanceof Throwable)) {
                 throw runtime.newTypeError("assigning non-exception to $!");
             }
-            
+
             return runtime.getCurrentContext().setErrorInfo(value);
         }
 
@@ -680,7 +685,7 @@ public class RubyGlobal {
             super(runtime, name, initialValue);
             set(initialValue);
         }
-        
+
         @Override
         public IRubyObject get() {
             return runtime.getVerbose();
@@ -697,7 +702,7 @@ public class RubyGlobal {
             return newValue;
         }
     }
-    
+
     private static class WarningGlobalVariable extends ReadonlyGlobalVariable {
         public WarningGlobalVariable(Ruby runtime, String name, RubyInstanceConfig.Verbosity verbosity) {
             super(runtime, name,
@@ -785,7 +790,7 @@ public class RubyGlobal {
             if (value == get()) {
                 return value;
             }
-            
+
             return super.set(value);
         }
     }
@@ -802,7 +807,7 @@ public class RubyGlobal {
             }
             if (value instanceof RubyIO) {
                 RubyIO io = (RubyIO)value;
-                
+
                 // HACK: in order to have stdout/err act like ttys and flush always,
                 // we set anything assigned to stdout/stderr to sync
                 try {
@@ -834,7 +839,7 @@ public class RubyGlobal {
         public LoadPath(Ruby runtime, String name) {
             super(runtime, name, null);
         }
-        
+
         /**
          * @see org.jruby.runtime.GlobalVariable#get()
          */
@@ -848,7 +853,7 @@ public class RubyGlobal {
         public LoadedFeatures(Ruby runtime, String name) {
             super(runtime, name, null);
         }
-        
+
         /**
          * @see org.jruby.runtime.GlobalVariable#get()
          */
