@@ -2,6 +2,7 @@
 #   e2mmap.rb - for ruby 1.1
 #   	$Release Version: 2.0$
 #   	$Revision: 1.10 $
+#   	$Date: 1999/02/17 12:33:17 $
 #   	by Keiju ISHITSUKA
 #
 # --
@@ -47,6 +48,8 @@
 #   Foo.Fail ExistingExceptionClass, arg...
 #
 #
+fail "Use Ruby 1.1" if VERSION < "1.1"
+
 module Exception2MessageMapper
   @RCS_ID='-$Id: e2mmap.rb,v 1.10 1999/02/17 12:33:17 keiju Exp keiju $-'
 
@@ -54,7 +57,13 @@ module Exception2MessageMapper
 
   def E2MM.extend_object(cl)
     super
-    cl.bind(self) unless cl < E2MM
+    cl.bind(self) unless cl == E2MM
+  end
+  
+  # backward compatibility
+  def E2MM.extend_to(b)
+    c = eval("self", b)
+    c.extend(self)
   end
 
   def bind(cl)
@@ -78,8 +87,21 @@ module Exception2MessageMapper
     E2MM.Raise(self, err, *rest)
   end
   alias Fail Raise
-  alias fail Raise
 
+  # backward compatibility
+  alias fail! fail
+  def fail(err = nil, *rest)
+    begin 
+      E2MM.Fail(self, err, *rest)
+    rescue E2MM::ErrNotRegisteredException
+      super
+    end
+  end
+  class << self
+    public :fail
+  end
+
+  
   # def_e2message(c, m)
   #	    c:  exception
   #	    m:  message_form
@@ -88,7 +110,7 @@ module Exception2MessageMapper
   def def_e2message(c, m)
     E2MM.def_e2message(self, c, m)
   end
-
+  
   # def_exception(n, m, s)
   #	    n:  exception_name
   #	    m:  message_form
@@ -105,7 +127,7 @@ module Exception2MessageMapper
   # {[class, exp] => message, ...}
   @MessageMap = {}
 
-  # E2MM.def_e2message(k, e, m)
+  # E2MM.def_exception(k, e, m)
   #	    k:  class to define exception under.
   #	    e:  exception
   #	    m:  message_form
@@ -115,7 +137,7 @@ module Exception2MessageMapper
     E2MM.instance_eval{@MessageMap[[k, c]] = m}
     c
   end
-
+  
   # E2MM.def_exception(k, n, m, s)
   #	    k:  class to define exception under.
   #	    n:  exception_name
@@ -137,11 +159,12 @@ module Exception2MessageMapper
   #
   def E2MM.Raise(klass = E2MM, err = nil, *rest)
     if form = e2mm_message(klass, err)
-      b = $@.nil? ? caller(1) : $@
+      $! = err.new(sprintf(form, *rest))
+      $@ = caller(1) if $@.nil?
       #p $@
       #p __FILE__
-      b.shift if b[0] =~ /^#{Regexp.quote(__FILE__)}:/
-      raise err, sprintf(form, *rest), b
+      $@.shift if $@[0] =~ /^#{Regexp.quote(__FILE__)}:/
+      raise
     else
       E2MM.Fail E2MM, ErrNotRegisteredException, err.inspect
     end
@@ -164,8 +187,8 @@ module Exception2MessageMapper
     alias message e2mm_message
   end
 
-  E2MM.def_exception(E2MM,
-		     :ErrNotRegisteredException,
+  E2MM.def_exception(E2MM, 
+		     :ErrNotRegisteredException, 
 		     "not registerd exception(%s)")
 end
 
