@@ -1,12 +1,13 @@
 #
-#   irb/lc/error.rb -
-#   	$Release Version: 0.9.6$
-#   	$Revision: 23985 $
+#   irb/lc/error.rb - 
+#   	$Release Version: 0.9.5$
+#   	$Revision$
+#   	$Date$
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
 #
-#
+#   
 #
 require "e2mmap"
 
@@ -17,7 +18,7 @@ module IRB
   def_exception :UnrecognizedSwitch, "Unrecognized switch: %s"
   def_exception :NotImplementedError, "Need to define `%s'"
   def_exception :CantReturnToNormalMode, "Can't return to normal mode."
-  def_exception :IllegalParameter, "Invalid parameter(%s)."
+  def_exception :IllegalParameter, "Illegal parameter(%s)."
   def_exception :IrbAlreadyDead, "Irb is already dead."
   def_exception :IrbSwitchedToCurrentThread, "Switched to current thread."
   def_exception :NoSuchJob, "No such job(%s)."
