@@ -70,6 +70,9 @@ module RSS
 
   class InvalidRSSError < Error; end
 
+  ##
+  # Raised if no matching tag is found.
+
   class MissingTagError < InvalidRSSError
     attr_reader :tag, :parent
     def initialize(tag, parent)
@@ -78,6 +81,9 @@ module RSS
     end
   end
 
+  ##
+  # Raised if there are more occurrences of the tag than expected.
+
   class TooMuchTagError < InvalidRSSError
     attr_reader :tag, :parent
     def initialize(tag, parent)
@@ -86,6 +92,9 @@ module RSS
     end
   end
 
+  ##
+  # Raised if a required attribute is missing.
+
   class MissingAttributeError < InvalidRSSError
     attr_reader :tag, :attribute
     def initialize(tag, attribute)
@@ -94,6 +103,9 @@ module RSS
     end
   end
 
+  ##
+  # Raised when an unknown tag is found.
+
   class UnknownTagError < InvalidRSSError
     attr_reader :tag, :uri
     def initialize(tag, uri)
@@ -102,6 +114,9 @@ module RSS
     end
   end
 
+  ##
+  # Raised when an unexpected tag is encountered.
+
   class NotExpectedTagError < InvalidRSSError
     attr_reader :tag, :uri, :parent
     def initialize(tag, uri, parent)
@@ -112,6 +127,9 @@ module RSS
   # For backward compatibility :X
   NotExceptedTagError = NotExpectedTagError
 
+  ##
+  # Raised when an incorrect value is used.
+
   class NotAvailableValueError < InvalidRSSError
     attr_reader :tag, :value, :attribute
     def initialize(tag, value, attribute=nil)
@@ -123,6 +141,9 @@ module RSS
     end
   end
 
+  ##
+  # Raised when an unknown conversion error occurs.
+
   class UnknownConversionMethodError < Error
     attr_reader :to, :from
     def initialize(to, from)
@@ -134,6 +155,9 @@ module RSS
   # for backward compatibility
   UnknownConvertMethod = UnknownConversionMethodError
 
+  ##
+  # Raised when a conversion failure occurs.
+
   class ConversionError < Error
     attr_reader :string, :to, :from
     def initialize(string, to, from)
@@ -144,6 +168,9 @@ module RSS
     end
   end
 
+  ##
+  # Raised when a required variable is not set.
+
   class NotSetError < Error
     attr_reader :name, :variables
     def initialize(name, variables)
@@ -153,6 +180,9 @@ module RSS
     end
   end
 
+  ##
+  # Raised when a RSS::Maker attempts to use an unknown maker.
+
   class UnsupportedMakerVersionError < Error
     attr_reader :version
     def initialize(version)
@@ -229,7 +259,7 @@ EOC
           else
             rv << value
           end
-    	    rv << "</#{elem_name}>"
+            rv << "</#{elem_name}>"
           rv
         else
           ''
@@ -260,7 +290,7 @@ EOC
           else
             rv << value
           end
-    	    rv << "</#{elem_name}>"
+            rv << "</#{elem_name}>"
           rv
         else
           ''
@@ -1093,9 +1123,8 @@ EOC
         tags = tags.sort_by {|x| element_names.index(x) || tags_size}
       end
 
-      _tags = tags.dup if tags
       models.each_with_index do |model, i|
-        name, model_uri, occurs, getter = model
+        name, _, occurs, = model
 
         if DEBUG
           p "before"
