@@ -115,6 +115,10 @@ module FileUtils
   #
   #   FileUtils.cd('/', :verbose => true)   # chdir and report it
   #
+  #   FileUtils.cd('/') do  # chdir
+  #     [...]               # do something
+  #   end                   # return to original directory
+  #
   def cd(dir, options = {}, &block) # :yield: dir
     fu_check_options options, OPT_TABLE['cd']
     fu_output_message "cd #{dir}" if options[:verbose]
@@ -138,9 +142,7 @@ module FileUtils
   #   FileUtils.uptodate?('hello.o', %w(hello.c hello.h)) or \
   #       system 'make hello.o'
   #
-  def uptodate?(new, old_list, options = nil)
-    raise ArgumentError, 'uptodate? does not accept any option' if options
-
+  def uptodate?(new, old_list)
     return false unless File.exist?(new)
     new_time = File.mtime(new)
     old_list.each do |old|
@@ -198,7 +200,7 @@ module FileUtils
     fu_output_message "mkdir -p #{options[:mode] ? ('-m %03o ' % options[:mode]) : ''}#{list.join ' '}" if options[:verbose]
     return *list if options[:noop]
 
-    list.map {|path| path.sub(%r</\z>, '') }.each do |path|
+    list.map {|path| path.chomp(?/) }.each do |path|
       # optimize for the most common case
       begin
         fu_mkdir path, options[:mode]
@@ -215,7 +217,7 @@ module FileUtils
       stack.reverse_each do |dir|
         begin
           fu_mkdir dir, options[:mode]
-        rescue SystemCallError => err
+        rescue SystemCallError
           raise unless File.directory?(dir)
         end
       end
@@ -235,7 +237,7 @@ module FileUtils
   OPT_TABLE['makedirs'] = [:mode, :noop, :verbose]
 
   def fu_mkdir(path, mode)   #:nodoc:
-    path = path.sub(%r</\z>, '')
+    path = path.chomp(?/)
     if mode
       Dir.mkdir path, mode
       File.chmod mode, path
@@ -263,13 +265,13 @@ module FileUtils
     return if options[:noop]
     list.each do |dir|
       begin
-        Dir.rmdir(dir = dir.sub(%r</\z>, ''))
+        Dir.rmdir(dir = dir.chomp(?/))
         if parents
           until (parent = File.dirname(dir)) == '.' or parent == dir
             Dir.rmdir(dir)
           end
         end
-      rescue Errno::ENOTEMPTY, Errno::ENOENT
+      rescue Errno::ENOTEMPTY, Errno::EEXIST, Errno::ENOENT
       end
     end
   end
@@ -359,7 +361,7 @@ module FileUtils
   # Options: noop verbose
   #
   # Same as
-  #   #ln_s(src, dest, :force)
+  #   #ln_s(src, dest, :force => true)
   #
   def ln_sf(src, dest, options = {})
     fu_check_options options, OPT_TABLE['ln_sf']
@@ -420,13 +422,15 @@ module FileUtils
   #   # If you want to copy all contents of a directory instead of the
   #   # directory itself, c.f. src/x -> dest/x, src/y -> dest/y,
   #   # use following code.
-  #   FileUtils.cp_r 'src/.', 'dest'     # cp_r('src', 'dest') makes src/dest,
+  #   FileUtils.cp_r 'src/.', 'dest'     # cp_r('src', 'dest') makes dest/src,
   #                                      # but this doesn't.
   #
   def cp_r(src, dest, options = {})
     fu_check_options options, OPT_TABLE['cp_r']
     fu_output_message "cp -r#{options[:preserve] ? 'p' : ''}#{options[:remove_destination] ? ' --remove-destination' : ''} #{[src,dest].flatten.join ' '}" if options[:verbose]
     return if options[:noop]
+    options = options.dup
+    options[:dereference_root] = true unless options.key?(:dereference_root)
     fu_each_src_dest(src, dest) do |s, d|
       copy_entry s, d, options[:preserve], options[:dereference_root], options[:remove_destination]
     end
@@ -445,20 +449,22 @@ module FileUtils
   # Both of +src+ and +dest+ must be a path name.
   # +src+ must exist, +dest+ must not exist.
   #
-  # If +preserve+ is true, this method preserves owner, group, permissions
-  # and modified time.
+  # If +preserve+ is true, this method preserves owner, group, and
+  # modified time.  Permissions are copied regardless +preserve+.
   #
   # If +dereference_root+ is true, this method dereference tree root.
   #
   # If +remove_destination+ is true, this method removes each destination file before copy.
   #
   def copy_entry(src, dest, preserve = false, dereference_root = false, remove_destination = false)
-    Entry_.new(src, nil, dereference_root).traverse do |ent|
+    Entry_.new(src, nil, dereference_root).wrap_traverse(proc do |ent|
       destent = Entry_.new(dest, ent.rel, false)
       File.unlink destent.path if remove_destination && File.file?(destent.path)
       ent.copy destent.path
+    end, proc do |ent|
+      destent = Entry_.new(dest, ent.rel, false)
       ent.copy_metadata destent.path if preserve
-    end
+    end)
   end
   module_function :copy_entry
 
@@ -511,7 +517,7 @@ module FileUtils
         end
         begin
           File.rename s, d
-        rescue Errno::EXDEV
+        rescue *MV_RESCUES
           copy_entry s, d, true
           if options[:secure]
             remove_entry_secure s, options[:force]
@@ -526,6 +532,15 @@ module FileUtils
   end
   module_function :mv
 
+  # JRuby raises EACCES because JDK reports errors differently
+  MV_RESCUES = begin
+    if RUBY_ENGINE == 'jruby'
+      [Errno::EXDEV, Errno::EACCES]
+    else
+      [Errno::EXDEV]
+    end
+  end
+
   alias move mv
   module_function :move
 
@@ -665,10 +680,10 @@ module FileUtils
   # removing directories.  This requires the current process is the
   # owner of the removing whole directory tree, or is the super user (root).
   #
-  # WARNING: You must ensure that *ALL* parent directories are not
-  # world writable.  Otherwise this method does not work.
-  # Only exception is temporary directory like /tmp and /var/tmp,
-  # whose permission is 1777.
+  # WARNING: You must ensure that *ALL* parent directories cannot be
+  # moved by other untrusted users.  For example, parent directories
+  # should not be owned by untrusted users, and should not be world
+  # writable except when the sticky bit set.
   #
   # WARNING: Only the owner of the removing directory tree, or Unix super
   # user (root) should invoke this method.  Otherwise this method does not
@@ -703,14 +718,20 @@ module FileUtils
     end
     # freeze tree root
     euid = Process.euid
-    File.open(fullpath + '/.') {|f|
-      unless fu_stat_identical_entry?(st, f.stat)
+    dot_file = fullpath + "/."
+    File.lstat(dot_file).tap {|fstat|
+      unless fu_stat_identical_entry?(st, fstat)
         # symlink (TOC-to-TOU attack?)
         File.unlink fullpath
         return
       end
-      f.chown euid, -1
-      f.chmod 0700
+      File.chown euid, -1, dot_file
+      File.chmod 0700, dot_file
+      unless fu_stat_identical_entry?(st, File.lstat(fullpath))
+        # TOC-to-TOU attack?
+        File.unlink fullpath
+        return
+      end
     }
     # ---- tree root is frozen ----
     root = Entry_.new(path)
@@ -732,11 +753,11 @@ module FileUtils
   end
   module_function :remove_entry_secure
 
-  def fu_have_symlink?   #:nodoc
+  def fu_have_symlink?   #:nodoc:
     File.symlink nil, nil
   rescue NotImplementedError
     return false
-  rescue
+  rescue TypeError
     return true
   end
   private_module_function :fu_have_symlink?
@@ -812,16 +833,13 @@ module FileUtils
   #
   def compare_stream(a, b)
     bsize = fu_stream_blksize(a, b)
-    sa = sb = nil
-    while sa == sb
-      sa = a.read(bsize)
-      sb = b.read(bsize)
-      unless sa and sb
-        if sa.nil? and sb.nil?
-          return true
-        end
-      end
-    end
+    sa = ""
+    sb = ""
+    begin
+      a.read(bsize, sa)
+      b.read(bsize, sb)
+      return true if sa.empty? && sb.empty?
+    end while sa == sb
     false
   end
   module_function :compare_stream
@@ -841,9 +859,9 @@ module FileUtils
     fu_output_message "install -c#{options[:preserve] && ' -p'}#{options[:mode] ? (' -m 0%o' % options[:mode]) : ''} #{[src,dest].flatten.join ' '}" if options[:verbose]
     return if options[:noop]
     fu_each_src_dest(src, dest) do |s, d|
+      st = File.stat(s)
       unless File.exist?(d) and compare_file(s, d)
         remove_file d, true
-        st = File.stat(s) if options[:preserve]
         copy_file s, d
         File.utime st.atime, st.mtime, d if options[:preserve]
         File.chmod options[:mode], d if options[:mode]
@@ -854,23 +872,117 @@ module FileUtils
 
   OPT_TABLE['install'] = [:mode, :preserve, :noop, :verbose]
 
+  def user_mask(target)  #:nodoc:
+    mask = 0
+    target.each_byte do |byte_chr|
+      case byte_chr.chr
+        when "u"
+          mask |= 04700
+        when "g"
+          mask |= 02070
+        when "o"
+          mask |= 01007
+        when "a"
+          mask |= 07777
+      end
+    end
+    mask
+  end
+  private_module_function :user_mask
+
+  def mode_mask(mode, path)  #:nodoc:
+    mask = 0
+    mode.each_byte do |byte_chr|
+      case byte_chr.chr
+        when "r"
+          mask |= 0444
+        when "w"
+          mask |= 0222
+        when "x"
+          mask |= 0111
+        when "X"
+          mask |= 0111 if FileTest::directory? path
+        when "s"
+          mask |= 06000
+        when "t"
+          mask |= 01000
+      end
+    end
+    mask
+  end
+  private_module_function :mode_mask
+
+  def symbolic_modes_to_i(modes, path)  #:nodoc:
+    current_mode = (File.stat(path).mode & 07777)
+    modes.split(/,/).inject(0) do |mode, mode_sym|
+      mode_sym = "a#{mode_sym}" if mode_sym =~ %r!^[=+-]!
+      target, mode = mode_sym.split %r![=+-]!
+      user_mask = user_mask(target)
+      mode_mask = mode_mask(mode ? mode : "", path)
+
+      case mode_sym
+        when /=/
+          current_mode &= ~(user_mask)
+          current_mode |= user_mask & mode_mask
+        when /\+/
+          current_mode |= user_mask & mode_mask
+        when /-/
+          current_mode &= ~(user_mask & mode_mask)
+      end
+    end
+  end
+  private_module_function :symbolic_modes_to_i
+
+  def fu_mode(mode, path)  #:nodoc:
+    mode.is_a?(String) ? symbolic_modes_to_i(mode, path) : mode
+  end
+  private_module_function :fu_mode
+
+  def mode_to_s(mode)  #:nodoc:
+    mode.is_a?(String) ? mode : "%o" % mode
+  end
+  private_module_function :mode_to_s
+
   #
   # Options: noop verbose
   #
   # Changes permission bits on the named files (in +list+) to the bit pattern
   # represented by +mode+.
   #
+  # +mode+ is the symbolic and absolute mode can be used.
+  #
+  # Absolute mode is
   #   FileUtils.chmod 0755, 'somecommand'
   #   FileUtils.chmod 0644, %w(my.rb your.rb his.rb her.rb)
   #   FileUtils.chmod 0755, '/usr/bin/ruby', :verbose => true
   #
+  # Symbolic mode is
+  #   FileUtils.chmod "u=wrx,go=rx", 'somecommand'
+  #   FileUtils.chmod "u=wr,go=rr", %w(my.rb your.rb his.rb her.rb)
+  #   FileUtils.chmod "u=wrx,go=rx", '/usr/bin/ruby', :verbose => true
+  #
+  # "a" :: is user, group, other mask.
+  # "u" :: is user's mask.
+  # "g" :: is group's mask.
+  # "o" :: is other's mask.
+  # "w" :: is write permission.
+  # "r" :: is read permission.
+  # "x" :: is execute permission.
+  # "X" ::
+  #   is execute permission for directories only, must be used in conjunction with "+"
+  # "s" :: is uid, gid.
+  # "t" :: is sticky bit.
+  # "+" :: is added to a class given the specified mode.
+  # "-" :: Is removed from a given class given mode.
+  # "=" :: Is the exact nature of the class will be given a specified mode.
+
   def chmod(mode, list, options = {})
     fu_check_options options, OPT_TABLE['chmod']
     list = fu_list(list)
-    fu_output_message sprintf('chmod %o %s', mode, list.join(' ')) if options[:verbose]
+    fu_output_message sprintf('chmod %s %s', mode_to_s(mode), list.join(' ')) if options[:verbose]
     return if options[:noop]
     list.each do |path|
-      Entry_.new(path).chmod mode
+      Entry_.new(path).chmod(fu_mode(mode, path))
     end
   end
   module_function :chmod
@@ -884,18 +996,19 @@ module FileUtils
   # to the bit pattern represented by +mode+.
   #
   #   FileUtils.chmod_R 0700, "/tmp/app.#{$$}"
+  #   FileUtils.chmod_R "u=wrx", "/tmp/app.#{$$}"
   #
   def chmod_R(mode, list, options = {})
     fu_check_options options, OPT_TABLE['chmod_R']
     list = fu_list(list)
-    fu_output_message sprintf('chmod -R%s %o %s',
+    fu_output_message sprintf('chmod -R%s %s %s',
                               (options[:force] ? 'f' : ''),
-                              mode, list.join(' ')) if options[:verbose]
+                              mode_to_s(mode), list.join(' ')) if options[:verbose]
     return if options[:noop]
     list.each do |root|
       Entry_.new(root).traverse do |ent|
         begin
-          ent.chmod mode
+          ent.chmod(fu_mode(mode, ent.path))
         rescue
           raise unless options[:force]
         end
@@ -921,8 +1034,8 @@ module FileUtils
   def chown(user, group, list, options = {})
     fu_check_options options, OPT_TABLE['chown']
     list = fu_list(list)
-    fu_output_message sprintf('chown %s%s',
-                              [user,group].compact.join(':') + ' ',
+    fu_output_message sprintf('chown %s %s',
+                              (group ? "#{user}:#{group}" : user || ':'),
                               list.join(' ')) if options[:verbose]
     return if options[:noop]
     uid = fu_get_uid(user)
@@ -950,9 +1063,9 @@ module FileUtils
   def chown_R(user, group, list, options = {})
     fu_check_options options, OPT_TABLE['chown_R']
     list = fu_list(list)
-    fu_output_message sprintf('chown -R%s %s%s',
+    fu_output_message sprintf('chown -R%s %s %s',
                               (options[:force] ? 'f' : ''),
-                              [user,group].compact.join(':') + ' ',
+                              (group ? "#{user}:#{group}" : user || ':'),
                               list.join(' ')) if options[:verbose]
     return if options[:noop]
     uid = fu_get_uid(user)
@@ -977,20 +1090,26 @@ module FileUtils
 
     def fu_get_uid(user)   #:nodoc:
       return nil unless user
-      user = user.to_s
-      if /\A\d+\z/ =~ user
-      then user.to_i
-      else Etc.getpwnam(user).uid
+      case user
+      when Integer
+        user
+      when /\A\d+\z/
+        user.to_i
+      else
+        Etc.getpwnam(user).uid
       end
     end
     private_module_function :fu_get_uid
 
     def fu_get_gid(group)   #:nodoc:
       return nil unless group
-      group = group.to_s
-      if /\A\d+\z/ =~ group
-      then group.to_i
-      else Etc.getgrnam(group).gid
+      case group
+      when Integer
+        group
+      when /\A\d+\z/
+        group.to_i
+      else
+        Etc.getgrnam(group).gid
       end
     end
     private_module_function :fu_get_gid
@@ -1024,7 +1143,7 @@ module FileUtils
     created = nocreate = options[:nocreate]
     t = options[:mtime]
     if options[:verbose]
-      fu_output_message "touch #{nocreate ? ' -c' : ''}#{t ? t.strftime(' -t %Y%m%d%H%M.%S') : ''}#{list.join ' '}"
+      fu_output_message "touch #{nocreate ? '-c ' : ''}#{t ? t.strftime('-t %Y%m%d%H%M.%S ') : ''}#{list.join ' '}"
     end
     return if options[:noop]
     list.each do |path|
@@ -1123,7 +1242,12 @@ module FileUtils
     end
 
     def exist?
-      lstat! ? true : false
+      begin
+        lstat
+        true
+      rescue Errno::ENOENT
+        false
+      end
     end
 
     def file?
@@ -1169,7 +1293,9 @@ module FileUtils
     end
 
     def entries
-      Dir.entries(path())\
+      opts = {}
+      opts[:encoding] = ::Encoding::UTF_8 if fu_windows?
+      Dir.entries(path(), opts)\
           .reject {|n| n == '.' or n == '..' }\
           .map {|n| Entry_.new(prefix(), join(rel(), n.untaint)) }
     end
@@ -1231,7 +1357,7 @@ module FileUtils
       when file?
         copy_file dest
       when directory?
-        if !File.exist?(dest) and /^#{Regexp.quote(path)}/ =~ File.dirname(dest)
+        if !File.exist?(dest) and descendant_diretory?(dest, path)
           raise ArgumentError, "cannot copy directory %s to itself %s" % [path, dest]
         end
         begin
@@ -1261,21 +1387,46 @@ module FileUtils
     end
 
     def copy_file(dest)
-      File.open(dest, 'wb') do |f|
-        IO.copy_stream(path(), f)
+      File.open(path()) do |s|
+        File.open(dest, 'wb', s.stat.mode) do |f|
+          IO.copy_stream(s, f)
+        end
       end
     end
 
     def copy_metadata(path)
       st = lstat()
-      File.utime st.atime, st.mtime, path
+      if !st.symlink?
+        File.utime st.atime, st.mtime, path
+      end
       begin
-        File.chown st.uid, st.gid, path
+        if st.symlink?
+          begin
+            File.lchown st.uid, st.gid, path
+          rescue NotImplementedError
+          end
+        else
+          File.chown st.uid, st.gid, path
+        end
       rescue Errno::EPERM
         # clear setuid/setgid
-        File.chmod st.mode & 01777, path
+        if st.symlink?
+          begin
+            File.lchmod st.mode & 01777, path
+          rescue NotImplementedError
+          end
+        else
+          File.chmod st.mode & 01777, path
+        end
       else
-        File.chmod st.mode, path
+        if st.symlink?
+          begin
+            File.lchmod st.mode, path
+          rescue NotImplementedError
+          end
+        else
+          File.chmod st.mode, path
+        end
       end
     end
 
@@ -1289,7 +1440,7 @@ module FileUtils
 
     def remove_dir1
       platform_support {
-        Dir.rmdir path().sub(%r</\z>, '')
+        Dir.rmdir path().chomp(?/)
       }
     end
 
@@ -1340,6 +1491,16 @@ module FileUtils
       yield self
     end
 
+    def wrap_traverse(pre, post)
+      pre.call self
+      if directory?
+        entries.each do |ent|
+          ent.wrap_traverse pre, post
+        end
+      end
+      post.call self
+    end
+
     private
 
     $fileutils_rb_have_lchmod = nil
@@ -1383,6 +1544,17 @@ module FileUtils
       return File.path(base) if not dir or dir == '.'
       File.join(dir, base)
     end
+
+    if File::ALT_SEPARATOR
+      DIRECTORY_TERM = "(?=[/#{Regexp.quote(File::ALT_SEPARATOR)}]|\\z)".freeze
+    else
+      DIRECTORY_TERM = "(?=/|\\z)".freeze
+    end
+    SYSCASE = File::FNM_SYSCASE.nonzero? ? "-i" : ""
+
+    def descendant_diretory?(descendant, ascendant)
+      /\A(?#{SYSCASE}:#{Regexp.quote(ascendant)})#{DIRECTORY_TERM}/ =~ File.dirname(descendant)
+    end
   end   # class Entry_
 
   def fu_list(arg)   #:nodoc:
@@ -1416,23 +1588,10 @@ module FileUtils
   private_module_function :fu_each_src_dest0
 
   def fu_same?(a, b)   #:nodoc:
-    if fu_have_st_ino?
-      st1 = File.stat(a)
-      st2 = File.stat(b)
-      st1.dev == st2.dev and st1.ino == st2.ino
-    else
-      File.expand_path(a) == File.expand_path(b)
-    end
-  rescue Errno::ENOENT
-    return false
+    File.identical?(a, b)
   end
   private_module_function :fu_same?
 
-  def fu_have_st_ino?   #:nodoc:
-    not fu_windows?
-  end
-  private_module_function :fu_have_st_ino?
-
   def fu_check_options(options, optdecl)   #:nodoc:
     h = options.dup
     optdecl.each do |opt|
@@ -1510,6 +1669,12 @@ module FileUtils
     OPT_TABLE.keys.select {|m| OPT_TABLE[m].include?(opt) }
   end
 
+  LOW_METHODS = singleton_methods(false) - collect_method(:noop).map(&:intern)
+  module LowMethods
+    module_eval("private\n" + ::FileUtils::LOW_METHODS.map {|name| "def #{name}(*)end"}.join("\n"),
+                __FILE__, __LINE__)
+  end
+
   METHODS = singleton_methods() - [:private_module_function,
       :commands, :options, :have_option?, :options_of, :collect_method]
 
@@ -1545,6 +1710,7 @@ module FileUtils
   #
   module NoWrite
     include FileUtils
+    include LowMethods
     @fileutils_output  = $stderr
     @fileutils_label   = ''
     ::FileUtils.collect_method(:noop).each do |name|
@@ -1571,6 +1737,7 @@ module FileUtils
   #
   module DryRun
     include FileUtils
+    include LowMethods
     @fileutils_output  = $stderr
     @fileutils_label   = ''
     ::FileUtils.collect_method(:noop).each do |name|
