@@ -6,12 +6,11 @@ begin
   import org.objectweb.asm.MethodVisitor
   import org.objectweb.asm.Opcodes
 rescue # jarjar renames things, so we try the renamed version
-  import "jruby.objectweb.asm.MethodVisitor"
-  import "jruby.objectweb.asm.Opcodes"
+  import "org.jruby.org.objectweb.asm.MethodVisitor"
+  import "org.jruby.org.objectweb.asm.Opcodes"
 end
 
 class MockMethodVisitor
-  include MethodVisitor
   attr_accessor :calls
 
   def initialize
@@ -25,36 +24,44 @@ end
 
 describe "SkinnyMethodAdapter" do  
   it "supports all JVM opcodes" do
-    keyword_opcodes = [] # gather opcodes that are named after keywords separately
     insn_opcodes = Opcodes.constants.select do |c|
-      case c
-      when /ACC_/ # access modifiers
-        false
-      when /V1_/ # version identifiers
-        false
-      when /T_/ # type identifiers
-        false
-      when /F_/ # framing hints
+      case c.to_s # for 1.9's symbols
+        
+      when /ACC_/, # access modifiers
+           /V1_/, # version identifiers
+           /T_/, # type identifiers
+           /F_/, # framing hints
+           /H_/, # method handles
+           /ASM/ # ASM version stuff
         false
+        
       when "DOUBLE", "FLOAT", "INTEGER", "LONG", "NULL", "TOP", "UNINITIALIZED_THIS"
         false
+        
       when "GOTO", "RETURN", "INSTANCEOF", "NEW"
         false
+        
       when "INVOKEDYNAMIC_OWNER"
         false
+        
       else
         true
       end
     end
     
+    # 1.9 makes them symbols, so to_s the lot
+    insn_opcodes = insn_opcodes.map(&:to_s)
+    instance_methods = SkinnyMethodAdapter.instance_methods.map(&:to_s)
+    
     insn_opcodes.each do |opcode|
       opcode = opcode.downcase
-      SkinnyMethodAdapter.instance_methods.should include(opcode)
+      instance_methods.should include(opcode)
     end
     
-    SkinnyMethodAdapter.instance_methods.should include("go_to")
-    SkinnyMethodAdapter.instance_methods.should include("voidreturn")
-    SkinnyMethodAdapter.instance_methods.should include("instance_of")
-    SkinnyMethodAdapter.instance_methods.should include("newobj")
+    instance_methods.should include("go_to")
+    instance_methods.should include("voidreturn")
+    instance_methods.should include("instance_of")
+    instance_methods.should include("newobj")
   end
-end
\ No newline at end of file
+end
+
