@@ -1,12 +1,15 @@
 require File.dirname(__FILE__) + "/../spec_helper"
 
-EXECUTOR_TEST_VALUE = 101
-
 describe "java.util.concurrent.Executors" do
+
+  EXECUTOR_TEST_VALUE = 101
+
   before do
     @executor = java.util.concurrent.Executors.newSingleThreadExecutor
   end
-  
+
+  after { @executor.shutdown }
+
   it "accepts a class that implements Callable interface" do
     cls = Class.new do
       include java.util.concurrent.Callable
@@ -15,12 +18,9 @@ describe "java.util.concurrent.Executors" do
         EXECUTOR_TEST_VALUE
       end
     end
-    lambda { @future = @executor.submit(cls.new) }.should_not raise_error(TypeError)
-    @future.get.should == EXECUTOR_TEST_VALUE
+    future = nil
+    lambda { future = @executor.submit(cls.new) }.should_not raise_error
+    future.get.should == EXECUTOR_TEST_VALUE
   end
-  
-  after do
-    @executor.shutdown
-  end
-  
+
 end
\ No newline at end of file
