@@ -1,7 +1,8 @@
 # specs for JRUBY-4680
 require File.dirname(__FILE__) + "/../spec_helper"
 
-import 'java_integration.fixtures.ClassWithMultipleSignaturesWithPrimitiveArgs'
+java_import 'java_integration.fixtures.ClassWithMultipleSignaturesWithPrimitiveArgs'
+java_import 'java_integration.fixtures.GenericComparable'
 
 describe "JRuby" do
   before :each do
@@ -17,3 +18,10 @@ describe "JRuby" do
     C.foo2(Object.new, @float).should == "float"
   end
 end
+
+describe "A class with two inexact overloads" do
+  it "will have the most specific overload called" do
+    GenericComparable.new.compareTo(0).should == 0
+    GenericComparable.new.compareTo('foo').should == 1
+  end
+end
\ No newline at end of file
