@@ -1,16 +1,6 @@
 require 'test/unit'
 
 class TestArray < Test::Unit::TestCase
-  def setup
-    @verbose = $VERBOSE
-    $VERBOSE = nil
-    @cls = Array
-  end
-
-  def teardown
-    $VERBOSE = @verbose
-  end
-
   def test_0_literal
     assert_equal([1, 2, 3, 4], [1, 2] + [3, 4])
     assert_equal([1, 2, 1, 2], [1, 2] * 2)
@@ -74,7 +64,7 @@ class TestArray < Test::Unit::TestCase
   end
 
   def test_split_0
-    x = "The Book of Mormon"
+    x = "The Boassert of Mormon"
     assert_equal(x.reverse, x.split(//).reverse!.join)
     assert_equal(x.reverse, x.reverse!)
     assert_equal("g:n:i:r:t:s: :e:t:y:b: :1", "1 byte string".split(//).reverse.join(":"))
@@ -125,27 +115,27 @@ class TestArray < Test::Unit::TestCase
 
   def test_beg_end_0
     x = [1, 2, 3, 4, 5]
-
+    
     assert_equal(1, x.first)
     assert_equal([1], x.first(1))
     assert_equal([1, 2, 3], x.first(3))
-
+    
     assert_equal(5, x.last)
     assert_equal([5], x.last(1))
     assert_equal([3, 4, 5], x.last(3))
-
+    
     assert_equal(1, x.shift)
     assert_equal([2, 3, 4], x.shift(3))
     assert_equal([5], x)
-
+    
     assert_equal([2, 3, 4, 5], x.unshift(2, 3, 4))
     assert_equal([1, 2, 3, 4, 5], x.unshift(1))
     assert_equal([1, 2, 3, 4, 5], x)
-
+    
     assert_equal(5, x.pop)
     assert_equal([3, 4], x.pop(2))
     assert_equal([1, 2], x)
-
+    
     assert_equal([1, 2, 3, 4], x.push(3, 4))
     assert_equal([1, 2, 3, 4, 5], x.push(5))
     assert_equal([1, 2, 3, 4, 5], x)
@@ -186,6 +176,10 @@ class TestArray < Test::Unit::TestCase
 
   # From rubicon
 
+  def setup
+    @cls = Array
+  end
+
   def test_00_new
     a = @cls.new()
     assert_instance_of(@cls, a)
@@ -318,7 +312,7 @@ class TestArray < Test::Unit::TestCase
 
     assert_equal(@cls[10, 11, 12], a[9..11])
     assert_equal(@cls[10, 11, 12], a[-91..-89])
-
+    
     assert_nil(a[10, -3])
     # Ruby 1.8 feature change:
     # Array#[size..x] returns [] instead of nil.
@@ -451,20 +445,16 @@ class TestArray < Test::Unit::TestCase
 
   def test_clone
     for taint in [ false, true ]
-      for untrust in [ false, true ]
-        for frozen in [ false, true ]
-          a = @cls[*(0..99).to_a]
-          a.taint  if taint
-          a.untrust  if untrust
-          a.freeze if frozen
-          b = a.clone
+      for frozen in [ false, true ]
+        a = @cls[*(0..99).to_a]
+        a.taint  if taint
+        a.freeze if frozen
+        b = a.clone
 
-          assert_equal(a, b)
-          assert(a.__id__ != b.__id__)
-          assert_equal(a.frozen?, b.frozen?)
-          assert_equal(a.untrusted?, b.untrusted?)
-          assert_equal(a.tainted?, b.tainted?)
-        end
+        assert_equal(a, b)
+        assert(a.__id__ != b.__id__)
+        assert_equal(a.frozen?, b.frozen?)
+        assert_equal(a.tainted?, b.tainted?)
       end
     end
   end
@@ -476,10 +466,7 @@ class TestArray < Test::Unit::TestCase
 
     assert_equal([], @cls[].collect { 99 })
 
-    # Ruby 1.9 feature change:
-    # Enumerable#collect without block returns an Enumerator.
-    #assert_equal([1, 2, 3], @cls[1, 2, 3].collect)
-    assert_kind_of Enumerator, @cls[1, 2, 3].collect
+    assert_equal([1, 2, 3], @cls[1, 2, 3].collect)
   end
 
   # also update map!
@@ -487,7 +474,7 @@ class TestArray < Test::Unit::TestCase
     a = @cls[ 1, 'cat', 1..1 ]
     assert_equal([ Fixnum, String, Range], a.collect! {|e| e.class} )
     assert_equal([ Fixnum, String, Range], a)
-
+   
     a = @cls[ 1, 'cat', 1..1 ]
     assert_equal([ 99, 99, 99], a.collect! { 99 } )
     assert_equal([ 99, 99, 99], a)
@@ -535,7 +522,7 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[1, 2, 3, 4],     @cls[1, 2, 3, 4].concat(@cls[]))
     assert_equal(@cls[],               @cls[].concat(@cls[]))
     assert_equal(@cls[@cls[1, 2], @cls[3, 4]], @cls[@cls[1, 2]].concat(@cls[@cls[3, 4]]))
-
+    
     a = @cls[1, 2, 3]
     a.concat(a)
     assert_equal([1, 2, 3, 1, 2, 3], a)
@@ -708,22 +695,10 @@ class TestArray < Test::Unit::TestCase
     a5 = @cls[ a1, @cls[], a3 ]
     assert_equal(@cls[1, 2, 3, 4, 5, 6], a5.flatten)
     assert_equal(@cls[], @cls[].flatten)
-    assert_equal(@cls[],
+    assert_equal(@cls[], 
                  @cls[@cls[@cls[@cls[],@cls[]],@cls[@cls[]],@cls[]],@cls[@cls[@cls[]]]].flatten)
 
     assert_raise(TypeError, "[ruby-dev:31197]") { [[]].flatten("") }
-
-    a6 = @cls[[1, 2], 3]
-    a6.taint
-    a6.untrust
-    a7 = a6.flatten
-    assert_equal(true, a7.tainted?)
-    assert_equal(true, a7.untrusted?)
-
-    a8 = @cls[[1, 2], 3]
-    a9 = a8.flatten(0)
-    assert_equal(a8, a9)
-    assert_not_same(a8, a9)
   end
 
   def test_flatten!
@@ -736,29 +711,27 @@ class TestArray < Test::Unit::TestCase
 
     a5 = @cls[ a1, @cls[], a3 ]
     assert_equal(@cls[1, 2, 3, 4, 5, 6], a5.flatten!)
-    assert_nil(a5.flatten!(0), '[ruby-core:23382]')
     assert_equal(@cls[1, 2, 3, 4, 5, 6], a5)
 
     assert_equal(@cls[], @cls[].flatten)
-    assert_equal(@cls[],
+    assert_equal(@cls[], 
                  @cls[@cls[@cls[@cls[],@cls[]],@cls[@cls[]],@cls[]],@cls[@cls[@cls[]]]].flatten)
-
-    assert_nil(@cls[].flatten!(0), '[ruby-core:23382]')
   end
 
-  def test_flatten_with_callcc
-    respond_to?(:callcc, true) or require 'continuation'
-    o = Object.new
-    def o.to_ary() callcc {|k| @cont = k; [1,2,3]} end
-    begin
-      assert_equal([10, 20, 1, 2, 3, 30, 1, 2, 3, 40], [10, 20, o, 30, o, 40].flatten)
-    rescue => e
-    else
-      o.instance_eval {@cont}.call
-    end
-    assert_instance_of(RuntimeError, e, '[ruby-dev:34798]')
-    assert_match(/reentered/, e.message, '[ruby-dev:34798]')
-  end
+  # JRuby does not support callcc
+  # def test_flatten_with_callcc
+  #   respond_to?(:callcc, true) or require 'continuation'
+  #   o = Object.new
+  #   def o.to_ary() callcc {|k| @cont = k; [1,2,3]} end
+  #   begin
+  #     assert_equal([10, 20, 1, 2, 3, 30, 1, 2, 3, 40], [10, 20, o, 30, o, 40].flatten)
+  #   rescue => e
+  #   else
+  #     o.instance_eval {@cont}.call
+  #   end
+  #   assert_instance_of(RuntimeError, e, '[ruby-dev:34798]')
+  #   assert_match(/reentered/, e.message, '[ruby-dev:34798]')
+  # end
 
   def test_hash
     a1 = @cls[ 'cat', 'dog' ]
@@ -785,8 +758,6 @@ class TestArray < Test::Unit::TestCase
     assert_equal(4, a.index([1,2,3]))
     assert_nil(a.index('ca'))
     assert_nil(a.index([1,2]))
-
-    assert_equal(1, a.index(99) {|x| x == 'cat' })
   end
 
   def test_values_at
@@ -818,14 +789,6 @@ class TestArray < Test::Unit::TestCase
     assert_equal("1,2,3", a.join(','))
 
     $, = ""
-    a = @cls[1, 2, 3]
-    a.taint
-    a.untrust
-    s = a.join
-    assert_equal(true, s.tainted?)
-    assert_equal(true, s.untrusted?)
-  ensure
-    $, = nil
   end
 
   def test_last
@@ -847,7 +810,7 @@ class TestArray < Test::Unit::TestCase
     a = @cls[ 1, 'cat', 1..1 ]
     assert_equal(@cls[ Fixnum, String, Range], a.map! {|e| e.class} )
     assert_equal(@cls[ Fixnum, String, Range], a)
-
+   
     a = @cls[ 1, 'cat', 1..1 ]
     assert_equal(@cls[ 99, 99, 99], a.map! { 99 } )
     assert_equal(@cls[ 99, 99, 99], a)
@@ -857,6 +820,14 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[], a)
   end
 
+  def test_nitems
+    assert_equal(0, @cls[].nitems)
+    assert_equal(1, @cls[1].nitems)
+    assert_equal(1, @cls[1, nil].nitems)
+    assert_equal(1, @cls[nil, 1].nitems)
+    assert_equal(3, @cls[1, nil, nil, 2, nil, 3, nil].nitems)
+  end
+
   def test_pack
     a = @cls[*%w( cat wombat x yy)]
     assert_equal("catwomx  yy ", a.pack("A3A3A3A3"))
@@ -888,17 +859,17 @@ class TestArray < Test::Unit::TestCase
     assert_equal("ABC",      @cls[ 65, 66, 67 ].pack("c3"))
     assert_equal("\377BC",   @cls[ -1, 66, 67 ].pack("c*"))
 
-
+    
     assert_equal("AB\n\x10",  @cls["4142", "0a", "12"].pack("H4H2H1"))
     assert_equal("AB\n\x02",  @cls["1424", "a0", "21"].pack("h4h2h1"))
 
-    assert_equal("abc=02def=\ncat=\n=01=\n",
+    assert_equal("abc=02def=\ncat=\n=01=\n", 
                  @cls["abc\002def", "cat", "\001"].pack("M9M3M4"))
 
     assert_equal("aGVsbG8K\n",  @cls["hello\n"].pack("m"))
     assert_equal(",:&5L;&\\*:&5L;&\\*\n",  @cls["hello\nhello\n"].pack("u"))
 
-    assert_equal("\u{a9 42 2260}", @cls[0xa9, 0x42, 0x2260].pack("U*"))
+    assert_equal("\xc2\xa9B\xe2\x89\xa0", @cls[0xa9, 0x42, 0x2260].pack("U*"))
 
 
     format = "c2x5CCxsdils_l_a6";
@@ -1042,8 +1013,6 @@ class TestArray < Test::Unit::TestCase
     assert_equal(4, a.rindex([1,2,3]))
     assert_nil(a.rindex('ca'))
     assert_nil(a.rindex([1,2]))
-
-    assert_equal(3, a.rindex(99) {|x| x == [1,2,3] })
   end
 
   def test_shift
@@ -1094,7 +1063,7 @@ class TestArray < Test::Unit::TestCase
 
     assert_equal(@cls[10, 11, 12], a.slice(9..11))
     assert_equal(@cls[10, 11, 12], a.slice(-91..-89))
-
+    
     assert_nil(a.slice(-101..-1))
 
     assert_nil(a.slice(10, -3))
@@ -1152,7 +1121,7 @@ class TestArray < Test::Unit::TestCase
 
     a.fill(1)
     assert_equal(@cls[1, 1, 1, 1], a.sort)
-
+    
     assert_equal(@cls[], @cls[].sort)
   end
 
@@ -1169,43 +1138,6 @@ class TestArray < Test::Unit::TestCase
 
     assert_equal(@cls[1], @cls[1].sort!)
     assert_equal(@cls[], @cls[].sort!)
-
-    a = @cls[4, 3, 2, 1]
-    a.sort! {|m, n| a.replace([9, 8, 7, 6]); m <=> n }
-    assert_equal([1, 2, 3, 4], a)
-
-    a = @cls[4, 3, 2, 1]
-    a.sort! {|m, n| a.replace([9, 8, 7]); m <=> n }
-    assert_equal([1, 2, 3, 4], a)
-  end
-
-  def test_sort_with_callcc
-    respond_to?(:callcc, true) or require 'continuation'
-    n = 1000
-    cont = nil
-    ary = (1..100).to_a
-    begin
-      ary.sort! {|a,b|
-        callcc {|k| cont = k} unless cont
-        assert_equal(100, ary.size, '[ruby-core:16679]')
-        a <=> b
-      }
-    rescue => e
-    end
-    n -= 1
-    cont.call if 0 < n
-    assert_instance_of(RuntimeError, e, '[ruby-core:16679]')
-    assert_match(/reentered/, e.message, '[ruby-core:16679]')
-  end
-
-  def test_sort_with_replace
-    xary = (1..100).to_a
-    100.times do
-      ary = (1..100).to_a
-      ary.sort! {|a,b| ary.replace(xary); a <=> b}
-      GC.start
-      assert_equal(xary, ary, '[ruby-dev:34732]')
-    end
   end
 
   def test_to_a
@@ -1229,21 +1161,21 @@ class TestArray < Test::Unit::TestCase
   def test_to_s
     $, = ""
     a = @cls[]
-    assert_equal("[]", a.to_s)
+    assert_equal("", a.to_s)
 
     $, = ""
     a = @cls[1, 2]
-    assert_equal("[1, 2]", a.to_s)
+    assert_equal("12", a.to_s)
 
     $, = ""
     a = @cls[1, 2, 3]
-    assert_equal("[1, 2, 3]", a.to_s)
+    assert_equal("123", a.to_s)
 
     $, = ":"
     a = @cls[1, 2, 3]
-    assert_equal("[1, 2, 3]", a.to_s)
-  ensure
-    $, = nil
+    assert_equal("1:2:3", a.to_s)
+
+    $, = ""
   end
 
   def test_uniq
@@ -1252,11 +1184,6 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[1, 2, 3, 4, nil], a.uniq)
     assert_equal(b, a)
 
-    c = @cls["a:def", "a:xyz", "b:abc", "b:xyz", "c:jkl"]
-    d = c.dup
-    assert_equal(@cls[ "a:def", "b:abc", "c:jkl" ], c.uniq {|s| s[/^\w+/]})
-    assert_equal(d, c)
-
     assert_equal(@cls[1, 2, 3], @cls[1, 2, 3].uniq)
   end
 
@@ -1265,14 +1192,6 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[1, 2, 3, 4, nil], a.uniq!)
     assert_equal(@cls[1, 2, 3, 4, nil], a)
 
-    c = @cls["a:def", "a:xyz", "b:abc", "b:xyz", "c:jkl"]
-    assert_equal(@cls[ "a:def", "b:abc", "c:jkl" ], c.uniq! {|s| s[/^\w+/]})
-    assert_equal(@cls[ "a:def", "b:abc", "c:jkl" ], c)
-
-    c = @cls["a:def", "b:abc", "c:jkl"]
-    assert_equal(nil, c.uniq! {|s| s[/^\w+/]})
-    assert_equal(@cls[ "a:def", "b:abc", "c:jkl" ], c)
-
     assert_nil(@cls[1, 2, 3].uniq!)
   end
 
@@ -1295,7 +1214,7 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[1,2], @cls[1, 2] | @cls[1, 2])
   end
 
-  def test_combination
+ def test_combination
     assert_equal(@cls[[]], @cls[1,2,3,4].combination(0).to_a)
     assert_equal(@cls[[1],[2],[3],[4]], @cls[1,2,3,4].combination(1).to_a)
     assert_equal(@cls[[1,2],[1,3],[1,4],[2,3],[2,4],[3,4]], @cls[1,2,3,4].combination(2).to_a)
@@ -1310,7 +1229,7 @@ class TestArray < Test::Unit::TestCase
     assert_equal(@cls[[1,1],[1,2],[2,1],[2,2]], @cls[1,2].product([1,2]))
 
     assert_equal(@cls[[1,3,5],[1,3,6],[1,4,5],[1,4,6],
-                   [2,3,5],[2,3,6],[2,4,5],[2,4,6]],
+                   [2,3,5],[2,3,6],[2,4,5],[2,4,6]], 
                  @cls[1,2].product([3,4],[5,6]))
     assert_equal(@cls[[1],[2]], @cls[1,2].product)
     assert_equal(@cls[], @cls[1,2].product([]))
@@ -1330,11 +1249,6 @@ class TestArray < Test::Unit::TestCase
                  "edcba".each_char.to_a.permutation(5).sort)
     assert_equal(@cls[].permutation(0).to_a, @cls[[]])
 
-    a = @cls[1, 2, 3, 4]
-    b = @cls[]
-    a.permutation {|x| b << x; a.replace(@cls[9, 8, 7, 6]) }
-    assert_equal(@cls[9, 8, 7, 6], a)
-    assert_equal(@cls[1, 2, 3, 4].permutation.to_a, b)
   end
 
   def test_take
@@ -1356,363 +1270,4 @@ class TestArray < Test::Unit::TestCase
   def test_drop_while
     assert_equal([3,4,5,0], [1,2,3,4,5,0].drop_while {|i| i < 3 })
   end
-
-  def test_modify_check
-    a = []
-    a.freeze
-    assert_raise(RuntimeError) { a.shift }
-    a = [1, 2]
-    assert_raise(SecurityError) do
-      Thread.new do
-        $SAFE = 4
-       a.shift
-      end.value
-    end
-  end
-
-  LONGP = [127, 63, 31, 15, 7].map {|x| 2**x-1 }.find do |x|
-    begin
-      [].first(x)
-    rescue ArgumentError
-      true
-    rescue RangeError
-      false
-    end
-  end
-
-  def test_ary_new
-    assert_raise(ArgumentError) { [].to_enum.first(-1) }
-    assert_raise(ArgumentError) { [].to_enum.first(LONGP) }
-  end
-
-  def test_try_convert
-    assert_equal([1], Array.try_convert([1]))
-    assert_equal(nil, Array.try_convert("1"))
-  end
-
-  def test_initialize
-    assert_nothing_raised { [].instance_eval { initialize } }
-    assert_nothing_raised { Array.new { } }
-    assert_equal([1, 2, 3], Array.new([1, 2, 3]))
-    assert_raise(ArgumentError) { Array.new(-1, 1) }
-    assert_raise(ArgumentError) { Array.new(LONGP, 1) }
-    assert_equal([1, 1, 1], Array.new(3, 1))
-    assert_equal([1, 1, 1], Array.new(3) { 1 })
-    assert_equal([1, 1, 1], Array.new(3, 1) { 1 })
-  end
-
-  def test_aset
-    assert_raise(IndexError) { [0][-2] = 1 }
-    assert_raise(IndexError) { [0][LONGP] = 2 }
-    assert_raise(IndexError) { [0][(LONGP + 1) / 2 - 1] = 2 }
-    a = [0]
-    a[2] = 4
-    assert_equal([0, nil, 4], a)
-    assert_raise(ArgumentError) { [0][0, 0, 0] = 0 }
-  end
-
-  def test_first2
-    assert_equal([0], [0].first(2))
-    assert_raise(ArgumentError) { [0].first(-1) }
-  end
-
-  def test_shift2
-    assert_equal(0, ([0] * 16).shift)
-    # check
-    a = [0, 1, 2]
-    a[3] = 3
-    a.shift(2)
-    assert_equal([2, 3], a)
-  end
-
-  def test_unshift2
-    Struct.new(:a, :b, :c)
-  end
-
-  def test_aref
-    assert_raise(ArgumentError) { [][0, 0, 0] }
-  end
-
-  def test_fetch
-    assert_equal(1, [].fetch(0, 0) { 1 })
-    assert_equal(1, [0, 1].fetch(-1))
-    assert_raise(IndexError) { [0, 1].fetch(2) }
-    assert_raise(IndexError) { [0, 1].fetch(-3) }
-    assert_equal(2, [0, 1].fetch(2, 2))
-  end
-
-  def test_index2
-    a = [0, 1, 2]
-    assert_equal(a, a.index.to_a)
-    assert_equal(1, a.index {|x| x == 1 })
-  end
-
-  def test_rindex2
-    a = [0, 1, 2]
-    assert_equal([2, 1, 0], a.rindex.to_a)
-    assert_equal(1, a.rindex {|x| x == 1 })
-
-    a = [0, 1]
-    e = a.rindex
-    assert_equal(1, e.next)
-    a.clear
-    assert_raise(StopIteration) { e.next }
-
-    o = Object.new
-    class << o; self; end.class_eval do
-      define_method(:==) {|x| a.clear; false }
-    end
-    a = [nil, o]
-    assert_equal(nil, a.rindex(0))
-  end
-
-  def test_ary_to_ary
-    o = Object.new
-    def o.to_ary; [1, 2, 3]; end
-    a, b, c = o
-    assert_equal([1, 2, 3], [a, b, c])
-  end
-
-  def test_splice
-    a = [0]
-    assert_raise(IndexError) { a[-2, 0] = nil }
-  end
-
-  def test_insert
-    a = [0]
-    assert_equal([0], a.insert(1))
-    assert_equal([0, 1], a.insert(1, 1))
-    assert_raise(ArgumentError) { a.insert }
-    assert_equal([0, 1, 2], a.insert(-1, 2))
-    assert_equal([0, 1, 3, 2], a.insert(-2, 3))
-  end
-
-  def test_join2
-    a = []
-    a << a
-    assert_raise(ArgumentError){a.join}
-
-    def (a = Object.new).to_a
-      [self]
-    end
-    assert_raise(ArgumentError, '[ruby-core:24150]'){[a].join}
-    assert_equal("12345", [1,[2,[3,4],5]].join)
-  end
-
-  def test_to_a2
-    klass = Class.new(Array)
-    a = klass.new.to_a
-    assert_equal([], a)
-    assert_equal(Array, a.class)
-  end
-
-  def test_values_at2
-    a = [0, 1, 2, 3, 4, 5]
-    assert_equal([1, 2, 3], a.values_at(1..3))
-    assert_equal([], a.values_at(7..8))
-    assert_equal([nil], a.values_at(2**31-1))
-  end
-
-  def test_select
-    assert_equal([0, 2], [0, 1, 2, 3].select {|x| x % 2 == 0 })
-  end
-
-  def test_delete2
-    a = [0] * 1024 + [1] + [0] * 1024
-    a.delete(0)
-    assert_equal([1], a)
-  end
-
-  def test_reject
-    assert_equal([1, 3], [0, 1, 2, 3].reject {|x| x % 2 == 0 })
-  end
-
-  def test_zip
-    assert_equal([[1, :a, "a"], [2, :b, "b"], [3, nil, "c"]],
-      [1, 2, 3].zip([:a, :b], ["a", "b", "c", "d"]))
-    a = []
-    [1, 2, 3].zip([:a, :b], ["a", "b", "c", "d"]) {|x| a << x }
-    assert_equal([[1, :a, "a"], [2, :b, "b"], [3, nil, "c"]], a)
-
-    ary = Object.new
-    def ary.to_a;   [1, 2]; end
-    assert_raise(NoMethodError){ %w(a b).zip(ary) }
-    def ary.each; [3, 4].each{|e|yield e}; end
-    assert_equal([['a', 3], ['b', 4]], %w(a b).zip(ary))
-    def ary.to_ary; [5, 6]; end
-    assert_equal([['a', 5], ['b', 6]], %w(a b).zip(ary))
-  end
-
-  def test_transpose
-    assert_equal([[1, :a], [2, :b], [3, :c]],
-      [[1, 2, 3], [:a, :b, :c]].transpose)
-    assert_raise(IndexError) { [[1, 2, 3], [:a, :b]].transpose }
-  end
-
-  def test_clear2
-    assert_equal([], ([0] * 1024).clear)
-  end
-
-  def test_fill2
-    assert_raise(ArgumentError) { [].fill(0, 1, LONGP) }
-  end
-
-  def test_times
-    assert_raise(ArgumentError) { [0, 0, 0, 0] * ((LONGP + 1) / 4) }
-  end
-
-  def test_equal
-    o = Object.new
-    def o.to_ary; end
-    def o.==(x); :foo; end
-    assert(:foo, [0, 1, 2] == o)
-    assert([0, 1, 2] != [0, 1, 3])
-  end
-
-  def test_hash2
-    a = []
-    a << a
-    assert_equal([[a]].hash, a.hash)
-    assert_not_equal([a, a].hash, a.hash) # Implementation dependent
-  end
-
-  def test_flatten2
-    a = []
-    a << a
-    assert_raise(ArgumentError) { a.flatten }
-  end
-
-  def test_shuffle
-    100.times do
-      assert_equal([0, 1, 2], [2, 1, 0].shuffle.sort)
-    end
-  end
-
-  def test_sample
-    100.times do
-      assert([0, 1, 2].include?([2, 1, 0].sample))
-      samples = [2, 1, 0].sample(2)
-      samples.each{|sample|
-        assert([0, 1, 2].include?(sample))
-      }
-    end
-
-    srand(0)
-    a = (1..18).to_a
-    (0..20).each do |n|
-      10000.times do
-        b = a.sample(n)
-        assert_equal([n, 18].min, b.uniq.size)
-        assert_equal(a, (a | b).sort)
-        assert_equal(b.sort, (a & b).sort)
-      end
-
-      h = Hash.new(0)
-      10000.times do
-        a.sample(n).each {|x| h[x] += 1 }
-      end
-      assert_operator(h.values.min * 2, :>=, h.values.max) if n != 0
-    end
-
-    assert_raise(ArgumentError, '[ruby-core:23374]') {[1, 2].sample(-1)}
-  end
-
-  def test_cycle
-    a = []
-    [0, 1, 2].cycle do |i|
-      a << i
-      break if a.size == 10
-    end
-    assert_equal([0, 1, 2, 0, 1, 2, 0, 1, 2, 0], a)
-
-    a = [0, 1, 2]
-    assert_nil(a.cycle { a.clear })
-
-    a = []
-    [0, 1, 2].cycle(3) {|i| a << i }
-    assert_equal([0, 1, 2, 0, 1, 2, 0, 1, 2], a)
-  end
-
-  def test_reverse_each2
-    a = [0, 1, 2, 3, 4, 5]
-    r = []
-    a.reverse_each do |x|
-      r << x
-      a.pop
-      a.pop
-    end
-    assert_equal([5, 3, 1], r)
-  end
-
-  def test_combination2
-    assert_raise(RangeError) do
-      (0..100).to_a.combination(50) {}
-    end
-  end
-
-  def test_product2
-    a = (0..100).to_a
-    assert_raise(RangeError) do
-      a.product(a, a, a, a, a, a, a, a, a, a) {}
-    end
-  end
-
-  class Array2 < Array
-  end
-
-  def test_array_subclass
-    assert_equal(Array2, Array2[1,2,3].uniq.class, "[ruby-dev:34581]")
-    assert_equal(Array2, Array2[1,2][0,1].class) # embeded
-    assert_equal(Array2, Array2[*(1..100)][1..99].class) #not embeded
-  end
-
-  def test_inspect
-    a = @cls[1, 2, 3]
-    a.taint
-    a.untrust
-    s = a.inspect
-    assert_equal(true, s.tainted?)
-    assert_equal(true, s.untrusted?)
-  end
-
-  def test_initialize2
-    a = [1] * 1000
-    a.instance_eval { initialize }
-    assert_equal([], a)
-  end
-
-  def test_shift_shared_ary
-    a = (1..100).to_a
-    b = []
-    b.replace(a)
-    assert_equal((1..10).to_a, a.shift(10))
-    assert_equal((11..100).to_a, a)
-  end
-
-  def test_replace_shared_ary
-    a = [1] * 100
-    b = []
-    b.replace(a)
-    a.replace([1, 2, 3])
-    assert_equal([1, 2, 3], a)
-    assert_equal([1] * 100, b)
-  end
-
-  def test_fill_negative_length
-    a = (1..10).to_a
-    a.fill(:foo, 5, -3)
-    assert_equal((1..10).to_a, a)
-  end
-
-  def test_slice_freezed_array
-    a = [1,2,3,4,5].freeze
-    assert_equal([1,2,3,4], a[0,4])
-    assert_equal([2,3,4,5], a[1,4])
-  end
-
-  def test_sort_by!
-    a = [1,3,5,2,4]
-    a.sort_by! {|x| -x }
-    assert_equal([5,4,3,2,1], a)
-  end
 end
