@@ -640,9 +640,7 @@ class TestDateParse < Test::Unit::TestCase
   end
 
   def test_parse_utf8
-    h = DateTime._parse(
-"Sun\xe3\x80\x80Aug 16 01:02:03 \xe6\x97\xa5\xe6\x9c\xac 2009".
-			force_encoding('utf-8'))
+    h = DateTime._parse("Sun\u{3000}Aug 16 01:02:03 \u{65e5}\u{672c} 2009")
     assert_equal(2009, h[:year])
     assert_equal(8, h[:mon])
     assert_equal(16, h[:mday])
@@ -650,14 +648,16 @@ class TestDateParse < Test::Unit::TestCase
     assert_equal(1, h[:hour])
     assert_equal(2, h[:min])
     assert_equal(3, h[:sec])
-    assert_equal("\xe6\x97\xa5\xe6\x9c\xac".
-		 force_encoding('utf-8'), h[:zone])
+    assert_equal("\u{65e5}\u{672c}", h[:zone])
   end
 
   def test_parse__ex
     assert_raise(ArgumentError) do
       Date.parse('')
     end
+    assert_raise(ArgumentError) do
+      DateTime.parse('')
+    end
     assert_raise(ArgumentError) do
       Date.parse('2001-02-29')
     end
@@ -670,14 +670,17 @@ class TestDateParse < Test::Unit::TestCase
   end
 
   def test__iso8601
-    h = Date._iso8601('01-02-03')
-    assert_equal([2001, 2, 3, nil, nil, nil, nil],
+    h = Date._iso8601('01-02-03T04:05:06Z')
+    assert_equal([2001, 2, 3, 4, 5, 6, 0],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('2001-02-03')
-    assert_equal([2001, 2, 3, nil, nil, nil, nil],
+    h = Date._iso8601('2001-02-03T04:05:06Z')
+    assert_equal([2001, 2, 3, 4, 5, 6, 0],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('--02-03')
-    assert_equal([nil, 2, 3, nil, nil, nil, nil],
+    h = Date._iso8601('--02-03T04:05:06Z')
+    assert_equal([nil, 2, 3, 4, 5, 6, 0],
+		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('---03T04:05:06Z')
+    assert_equal([nil, nil, 3, 4, 5, 6, 0],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
 
     h = Date._iso8601('2001-02-03T04:05')
@@ -696,14 +699,17 @@ class TestDateParse < Test::Unit::TestCase
     assert_equal([2001, 2, 3, 4, 5, 6, 3600],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
 
-    h = Date._iso8601('010203')
-    assert_equal([2001, 2, 3, nil, nil, nil, nil],
+    h = Date._iso8601('010203T040506Z')
+    assert_equal([2001, 2, 3, 4, 5, 6, 0],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('20010203')
-    assert_equal([2001, 2, 3, nil, nil, nil, nil],
+    h = Date._iso8601('20010203T040506Z')
+    assert_equal([2001, 2, 3, 4, 5, 6, 0],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('--0203')
-    assert_equal([nil, 2, 3, nil, nil, nil, nil],
+    h = Date._iso8601('--0203T040506Z')
+    assert_equal([nil, 2, 3, 4, 5, 6, 0],
+		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('---03T040506Z')
+    assert_equal([nil, nil, 3, 4, 5, 6, 0],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
 
     h = Date._iso8601('010203T0405')
@@ -741,16 +747,52 @@ class TestDateParse < Test::Unit::TestCase
     assert_equal([2001, 2, 3, 4, 5, 6, 3600],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
 
-    h = Date._iso8601('01-023')
-    assert_equal([2001, 23, nil, nil, nil, nil],
+    h = Date._iso8601('01-023T04:05:06Z')
+    assert_equal([2001, 23, 4, 5, 6, 0],
 		 h.values_at(:year, :yday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('2001-023')
-    assert_equal([2001, 23, nil, nil, nil, nil],
+    h = Date._iso8601('2001-023T04:05:06Z')
+    assert_equal([2001, 23, 4, 5, 6, 0],
 		 h.values_at(:year, :yday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('-023')
-    assert_equal([nil, 23, nil, nil, nil, nil],
+    h = Date._iso8601('-023T04:05:06Z')
+    assert_equal([nil, 23, 4, 5, 6, 0],
 		 h.values_at(:year, :yday, :hour, :min, :sec, :offset))
 
+    h = Date._iso8601('01023T040506Z')
+    assert_equal([2001, 23, 4, 5, 6, 0],
+		 h.values_at(:year, :yday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('2001023T040506Z')
+    assert_equal([2001, 23, 4, 5, 6, 0],
+		 h.values_at(:year, :yday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('-023T040506Z')
+    assert_equal([nil, 23, 4, 5, 6, 0],
+		 h.values_at(:year, :yday, :hour, :min, :sec, :offset))
+
+    h = Date._iso8601('01-w02-3T04:05:06Z')
+    assert_equal([2001, 2, 3, 4, 5, 6, 0],
+		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('2001-w02-3T04:05:06Z')
+    assert_equal([2001, 2, 3, 4, 5, 6, 0],
+		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('-w02-3T04:05:06Z')
+    assert_equal([nil, 2, 3, 4, 5, 6, 0],
+		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('-w-3T04:05:06Z')
+    assert_equal([nil, nil, 3, 4, 5, 6, 0],
+		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
+
+    h = Date._iso8601('01w023T040506Z')
+    assert_equal([2001, 2, 3, 4, 5, 6, 0],
+		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('2001w023T040506Z')
+    assert_equal([2001, 2, 3, 4, 5, 6, 0],
+		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('-w023T040506Z')
+    assert_equal([nil, 2, 3, 4, 5, 6, 0],
+		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
+    h = Date._iso8601('-w-3T040506Z')
+    assert_equal([nil, nil, 3, 4, 5, 6, 0],
+		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
+
     h = Date._iso8601('04:05')
     assert_equal([nil, nil, nil, 4, 5, nil, nil],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
@@ -774,21 +816,6 @@ class TestDateParse < Test::Unit::TestCase
     assert_equal([nil, nil, nil, 4, 5, 6, 3600],
 		 h.values_at(:year, :mon, :mday, :hour, :min, :sec, :offset))
 
-    h = Date._iso8601('01-w02-3')
-    assert_equal([2001, 2, 3, nil, nil, nil, nil],
-		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('2001-w02-3')
-    assert_equal([2001, 2, 3, nil, nil, nil, nil],
-		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('2001w023')
-    assert_equal([2001, 2, 3, nil, nil, nil, nil],
-		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('-w02-3')
-    assert_equal([nil, 2, 3, nil, nil, nil, nil],
-		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
-    h = Date._iso8601('-w-3')
-    assert_equal([nil, nil, 3, nil, nil, nil, nil],
-		 h.values_at(:cwyear, :cweek, :cwday, :hour, :min, :sec, :offset))
   end
 
   def test__rfc3339
@@ -956,16 +983,40 @@ class TestDateParse < Test::Unit::TestCase
   def test_iso8601
     assert_instance_of(Date, Date.iso8601)
     assert_instance_of(DateTime, DateTime.iso8601)
+
+    d = Date.iso8601('2001-02-03', Date::ITALY + 10)
+    assert_equal(Date.new(2001,2,3), d)
+    assert_equal(Date::ITALY + 10, d.start)
+
+    d = DateTime.iso8601('2001-02-03T04:05:06+07:00', Date::ITALY + 10)
+    assert_equal(DateTime.new(2001,2,3,4,5,6,'+07:00'), d)
+    assert_equal(Date::ITALY + 10, d.start)
   end
 
   def test_rfc3339
     assert_instance_of(Date, Date.rfc3339)
     assert_instance_of(DateTime, DateTime.rfc3339)
+
+    d = Date.rfc3339('2001-02-03T04:05:06+07:00', Date::ITALY + 10)
+    assert_equal(Date.new(2001,2,3), d)
+    assert_equal(Date::ITALY + 10, d.start)
+
+    d = DateTime.rfc3339('2001-02-03T04:05:06+07:00', Date::ITALY + 10)
+    assert_equal(DateTime.new(2001,2,3,4,5,6,'+07:00'), d)
+    assert_equal(Date::ITALY + 10, d.start)
   end
 
   def test_xmlschema
     assert_instance_of(Date, Date.xmlschema)
     assert_instance_of(DateTime, DateTime.xmlschema)
+
+    d = Date.xmlschema('2001-02-03', Date::ITALY + 10)
+    assert_equal(Date.new(2001,2,3), d)
+    assert_equal(Date::ITALY + 10, d.start)
+
+    d = DateTime.xmlschema('2001-02-03T04:05:06+07:00', Date::ITALY + 10)
+    assert_equal(DateTime.new(2001,2,3,4,5,6,'+07:00'), d)
+    assert_equal(Date::ITALY + 10, d.start)
   end
 
   def test_rfc2822
@@ -973,16 +1024,74 @@ class TestDateParse < Test::Unit::TestCase
     assert_instance_of(DateTime, DateTime.rfc2822)
     assert_instance_of(Date, Date.rfc822)
     assert_instance_of(DateTime, DateTime.rfc822)
+
+    d = Date.rfc2822('Sat, 3 Feb 2001 04:05:06 +0700', Date::ITALY + 10)
+    assert_equal(Date.new(2001,2,3), d)
+    assert_equal(Date::ITALY + 10, d.start)
+
+    d = DateTime.rfc2822('Sat, 3 Feb 2001 04:05:06 +0700', Date::ITALY + 10)
+    assert_equal(DateTime.new(2001,2,3,4,5,6,'+07:00'), d)
+    assert_equal(Date::ITALY + 10, d.start)
   end
 
   def test_httpdate
     assert_instance_of(Date, Date.httpdate)
     assert_instance_of(DateTime, DateTime.httpdate)
+
+    d = Date.httpdate('Sat, 03 Feb 2001 04:05:06 GMT', Date::ITALY + 10)
+    assert_equal(Date.new(2001,2,3), d)
+    assert_equal(Date::ITALY + 10, d.start)
+
+    d = DateTime.httpdate('Sat, 03 Feb 2001 04:05:06 GMT', Date::ITALY + 10)
+    assert_equal(DateTime.new(2001,2,3,4,5,6,'+00:00'), d)
+    assert_equal(Date::ITALY + 10, d.start)
   end
 
   def test_jisx0301
     assert_instance_of(Date, Date.jisx0301)
     assert_instance_of(DateTime, DateTime.jisx0301)
+
+    d = Date.jisx0301('H13.02.03', Date::ITALY + 10)
+    assert_equal(Date.new(2001,2,3), d)
+    assert_equal(Date::ITALY + 10, d.start)
+
+    d = DateTime.jisx0301('H13.02.03T04:05:06+07:00', Date::ITALY + 10)
+    assert_equal(DateTime.new(2001,2,3,4,5,6,'+07:00'), d)
+    assert_equal(Date::ITALY + 10, d.start)
+  end
+
+  def test_given_string
+    s = '2001-02-03T04:05:06Z'
+    s0 = s.dup
+
+    assert_not_equal({}, Date._parse(s))
+    assert_equal(s0, s)
+
+    assert_not_equal({}, Date._iso8601(s))
+    assert_equal(s0, s)
+
+    assert_not_equal({}, Date._rfc3339(s))
+    assert_equal(s0, s)
+
+    assert_not_equal({}, Date._xmlschema(s))
+    assert_equal(s0, s)
+
+    s = 'Sat, 3 Feb 2001 04:05:06 UT'
+    s0 = s.dup
+    assert_not_equal({}, Date._rfc2822(s))
+    assert_equal(s0, s)
+    assert_not_equal({}, Date._rfc822(s))
+    assert_equal(s0, s)
+
+    s = 'Sat, 03 Feb 2001 04:05:06 GMT'
+    s0 = s.dup
+    assert_not_equal({}, Date._httpdate(s))
+    assert_equal(s0, s)
+
+    s = 'H13.02.03T04:05:06,07Z'
+    s0 = s.dup
+    assert_not_equal({}, Date._jisx0301(s))
+    assert_equal(s0, s)
   end
 
 end
