@@ -1,4 +1,5 @@
 require 'test/unit'
+require 'tmpdir'
 
 begin
   require 'gdbm'
@@ -9,10 +10,10 @@ if defined? GDBM
   require 'tmpdir'
   require 'fileutils'
 
-  class TestGDBM < Test::Unit::TestCase
-    def TestGDBM.uname_s
+  class TestGDBM_RDONLY < Test::Unit::TestCase
+    def TestGDBM_RDONLY.uname_s
       require 'rbconfig'
-      case Config::CONFIG['target_os']
+      case RbConfig::CONFIG['target_os']
       when 'cygwin'
         require 'Win32API'
         uname = Win32API.new('cygwin1', 'uname', 'P', 'I')
@@ -21,29 +22,56 @@ if defined? GDBM
 
         utsname.unpack('A20' * 5)[0]
       else
-        Config::CONFIG['target_os']
+        RbConfig::CONFIG['target_os']
       end
     end
     SYSTEM = uname_s
 
     def setup
-      @path = "tmptest_gdbm_"
-      assert_instance_of(GDBM, @gdbm = GDBM.new(@path))
+      @tmpdir = Dir.mktmpdir("tmptest_gdbm")
+      @prefix = "tmptest_gdbm_#{$$}"
+      @path = "#{@tmpdir}/#{@prefix}_"
 
       # prepare to make readonly GDBM file
-      GDBM.open("tmptest_gdbm_rdonly", 0400) {|gdbm|
+      GDBM.open("#{@tmpdir}/#{@prefix}_rdonly", 0400) {|gdbm|
         gdbm['foo'] = 'FOO'
       }
-      assert_instance_of(GDBM, @gdbm_rdonly = GDBM.new("tmptest_gdbm_rdonly", nil))
+      assert_instance_of(GDBM, @gdbm_rdonly = GDBM.new("#{@tmpdir}/#{@prefix}_rdonly", nil))
     end
     def teardown
-      assert_nil(@gdbm.close)
       assert_nil(@gdbm_rdonly.close)
       ObjectSpace.each_object(GDBM) do |obj|
         obj.close unless obj.closed?
       end
-      File.delete *Dir.glob("*tmptest_gdbm*").to_a
-      p Dir.glob("*tmptest_gdbm*") if $DEBUG
+      FileUtils.remove_entry_secure @tmpdir
+    end
+
+    def test_delete_rdonly
+      if /^CYGWIN_9/ !~ SYSTEM
+        assert_raise(GDBMError) {
+          @gdbm_rdonly.delete("foo")
+        }
+
+        assert_nil(@gdbm_rdonly.delete("bar"))
+      end
+    end
+  end
+
+  class TestGDBM < Test::Unit::TestCase
+    SYSTEM = TestGDBM_RDONLY::SYSTEM
+
+    def setup
+      @tmpdir = Dir.mktmpdir("tmptest_gdbm")
+      @prefix = "tmptest_gdbm_#{$$}"
+      @path = "#{@tmpdir}/#{@prefix}_"
+      assert_instance_of(GDBM, @gdbm = GDBM.new(@path))
+    end
+    def teardown
+      assert_nil(@gdbm.close)
+      ObjectSpace.each_object(GDBM) do |obj|
+        obj.close unless obj.closed?
+      end
+      FileUtils.remove_entry_secure @tmpdir
     end
 
     def check_size(expect, gdbm=@gdbm)
@@ -70,7 +98,7 @@ if defined? GDBM
     def test_s_new_has_no_block
       # GDBM.new ignore the block
       foo = true
-      assert_instance_of(GDBM, gdbm = GDBM.new("tmptest_gdbm") { foo = false })
+      assert_instance_of(GDBM, gdbm = GDBM.new("#{@tmpdir}/#{@prefix}") { foo = false })
       assert_equal(foo, true)
       assert_nil(gdbm.close)
     end
@@ -79,54 +107,54 @@ if defined? GDBM
 
       save_mask = File.umask(0)
       begin
-        assert_instance_of(GDBM, gdbm = GDBM.open("tmptest_gdbm"))
+        assert_instance_of(GDBM, gdbm = GDBM.open("#{@tmpdir}/#{@prefix}"))
         gdbm.close
-        assert_equal(File.stat("tmptest_gdbm").mode & 0777, 0666)
-        assert_instance_of(GDBM, gdbm = GDBM.open("tmptest_gdbm2", 0644))
+        assert_equal(File.stat("#{@tmpdir}/#{@prefix}").mode & 0777, 0666) unless /mswin|mingw/ =~ RUBY_PLATFORM
+        assert_instance_of(GDBM, gdbm = GDBM.open("#{@tmpdir}/#{@prefix}2", 0644))
         gdbm.close
-        assert_equal(File.stat("tmptest_gdbm2").mode & 0777, 0644)
+        assert_equal(File.stat("#{@tmpdir}/#{@prefix}2").mode & 0777, 0644)
       ensure
         File.umask save_mask
       end
     end
     def test_s_open_no_create
-      assert_nil(gdbm = GDBM.open("tmptest_gdbm", nil),
-                 "this test is failed on libgdbm 1.8.0")
+      skip "gdbm_open(GDBM_WRITER) is broken on libgdbm 1.8.0" if /1\.8\.0/ =~ GDBM::VERSION
+      assert_nil(gdbm = GDBM.open("#{@tmpdir}/#{@prefix}", nil))
     ensure
       gdbm.close if gdbm
     end
     def test_s_open_3rd_arg
-      assert_instance_of(GDBM, gdbm = GDBM.open("tmptest_gdbm", 0644,
+      assert_instance_of(GDBM, gdbm = GDBM.open("#{@tmpdir}/#{@prefix}", 0644,
                                                 GDBM::FAST))
       gdbm.close
 
       # gdbm 1.8.0 specific
       if defined? GDBM::SYNC
-        assert_instance_of(GDBM, gdbm = GDBM.open("tmptest_gdbm", 0644,
+        assert_instance_of(GDBM, gdbm = GDBM.open("#{@tmpdir}/#{@prefix}", 0644,
                                                   GDBM::SYNC))
         gdbm.close
       end
       # gdbm 1.8.0 specific
       if defined? GDBM::NOLOCK
-        assert_instance_of(GDBM, gdbm = GDBM.open("tmptest_gdbm", 0644,
+        assert_instance_of(GDBM, gdbm = GDBM.open("#{@tmpdir}/#{@prefix}", 0644,
                                                   GDBM::NOLOCK))
         gdbm.close
       end
     end
     def test_s_open_with_block
-      assert_equal(GDBM.open("tmptest_gdbm") { :foo }, :foo)
+      assert_equal(GDBM.open("#{@tmpdir}/#{@prefix}") { :foo }, :foo)
     end
     def test_s_open_lock
       return unless have_fork?	# snip this test
       pid = fork() {
-        assert_instance_of(GDBM, gdbm  = GDBM.open("tmptest_gdbm", 0644))
+        assert_instance_of(GDBM, gdbm  = GDBM.open("#{@tmpdir}/#{@prefix}", 0644))
         sleep 2
       }
       begin
         sleep 1
         assert_raise(Errno::EWOULDBLOCK) {
           begin
-            assert_instance_of(GDBM, gdbm2 = GDBM.open("tmptest_gdbm", 0644))
+            assert_instance_of(GDBM, gdbm2 = GDBM.open("#{@tmpdir}/#{@prefix}", 0644))
           rescue Errno::EAGAIN, Errno::EACCES
             raise Errno::EWOULDBLOCK
           end
@@ -140,10 +168,10 @@ if defined? GDBM
     # Is it guaranteed on many OS?
     def test_s_open_lock_one_process
       # locking on one process
-      assert_instance_of(GDBM, gdbm  = GDBM.open("tmptest_gdbm", 0644))
+      assert_instance_of(GDBM, gdbm  = GDBM.open("#{@tmpdir}/#{@prefix}", 0644))
       assert_raise(Errno::EWOULDBLOCK) {
         begin
-          GDBM.open("tmptest_gdbm", 0644)
+          GDBM.open("#{@tmpdir}/#{@prefix}", 0644)
         rescue Errno::EAGAIN
           raise Errno::EWOULDBLOCK
         end
@@ -159,7 +187,7 @@ if defined? GDBM
       return unless have_fork?	# snip this test
 
       pid = fork() {
-        assert_instance_of(GDBM, gdbm  = GDBM.open("tmptest_gdbm", 0644,
+        assert_instance_of(GDBM, gdbm  = GDBM.open("#{@tmpdir}/#{@prefix}", 0644,
                                                   GDBM::NOLOCK))
         sleep 2
       }
@@ -167,17 +195,17 @@ if defined? GDBM
       begin
         gdbm2 = nil
         assert_nothing_raised(Errno::EWOULDBLOCK, Errno::EAGAIN, Errno::EACCES) {
-          assert_instance_of(GDBM, gdbm2 = GDBM.open("tmptest_gdbm", 0644))
+          assert_instance_of(GDBM, gdbm2 = GDBM.open("#{@tmpdir}/#{@prefix}", 0644))
         }
       ensure
         Process.wait pid
         gdbm2.close if gdbm2
       end
 
-      p Dir.glob("tmptest_gdbm*") if $DEBUG
+      p Dir.glob("#{@tmpdir}/#{@prefix}*") if $DEBUG
 
-      pid = Process.fork() {
-        assert_instance_of(GDBM, gdbm  = GDBM.open("tmptest_gdbm", 0644))
+      pid = fork() {
+        assert_instance_of(GDBM, gdbm  = GDBM.open("#{@tmpdir}/#{@prefix}", 0644))
         sleep 2
       }
       begin
@@ -185,7 +213,7 @@ if defined? GDBM
         gdbm2 = nil
         assert_nothing_raised(Errno::EWOULDBLOCK, Errno::EAGAIN, Errno::EACCES) {
           # this test is failed on Cygwin98 (???)
-          assert_instance_of(GDBM, gdbm2 = GDBM.open("tmptest_gdbm", 0644,
+          assert_instance_of(GDBM, gdbm2 = GDBM.open("#{@tmpdir}/#{@prefix}", 0644,
                                                      GDBM::NOLOCK))
         }
       ensure
@@ -195,16 +223,15 @@ if defined? GDBM
     end
 
     def test_s_open_error
-      return if /(ms|bcc)win|mingw|djgpp/ =~ RUBY_PLATFORM
-      assert_instance_of(GDBM, gdbm = GDBM.open("tmptest_gdbm", 0))
-      assert_raise(Errno::EACCES) {
-        GDBM.open("tmptest_gdbm", 0)
+      assert_instance_of(GDBM, gdbm = GDBM.open("#{@tmpdir}/#{@prefix}", 0))
+      assert_raise(Errno::EACCES, Errno::EWOULDBLOCK) {
+        GDBM.open("#{@tmpdir}/#{@prefix}", 0)
       }
       gdbm.close
     end
 
     def test_close
-      assert_instance_of(GDBM, gdbm = GDBM.open("tmptest_gdbm"))
+      assert_instance_of(GDBM, gdbm = GDBM.open("#{@tmpdir}/#{@prefix}"))
       assert_nil(gdbm.close)
 
       # closed GDBM file
@@ -275,9 +302,9 @@ if defined? GDBM
       }
     end
 
-    def test_index
+    def test_key
       assert_equal('bar', @gdbm['foo'] = 'bar')
-      assert_equal('foo', @gdbm.index('bar'))
+      assert_equal('foo', @gdbm.key('bar'))
       assert_nil(@gdbm['bar'])
     end
 
@@ -353,7 +380,7 @@ if defined? GDBM
 
       n = 0
       ret = @gdbm.each_value {|val|
-        assert_not_nil(key = @gdbm.index(val))
+        assert_not_nil(key = @gdbm.key(val))
         assert_not_nil(i = keys.index(key))
         assert_equal(val, values[i])
 
@@ -437,15 +464,8 @@ if defined? GDBM
       assert_equal(2, @gdbm.size)
 
       assert_nil(@gdbm.delete(key))
-
-      if /^CYGWIN_9/ !~ SYSTEM
-        assert_raise(GDBMError) {
-          @gdbm_rdonly.delete("foo")
-        }
-
-        assert_nil(@gdbm_rdonly.delete("bar"))
-      end
     end
+
     def test_delete_with_block
       key = 'no called block'
       @gdbm[key] = 'foo'
@@ -593,10 +613,10 @@ if defined? GDBM
     end
 
     def test_sync
-      assert_instance_of(GDBM, gdbm = GDBM.open('tmptest_gdbm', 0666, GDBM::FAST))
+      assert_instance_of(GDBM, gdbm = GDBM.open("#{@tmpdir}/#{@prefix}", 0666, GDBM::FAST))
       assert_equal(gdbm.sync, gdbm)
       gdbm.close
-      assert_instance_of(GDBM, gdbm = GDBM.open('tmptest_gdbm', 0666))
+      assert_instance_of(GDBM, gdbm = GDBM.open("#{@tmpdir}/#{@prefix}", 0666))
       assert_equal(gdbm.sync, gdbm)
       gdbm.close
     end
@@ -651,30 +671,54 @@ if defined? GDBM
   end
 
   class TestGDBM2 < Test::Unit::TestCase
-    TMPROOT = "#{Dir.tmpdir}/ruby-gdbm.#{$$}"
-
     def setup
-      Dir.mkdir TMPROOT
+      @tmproot = Dir.mktmpdir('ruby-gdbm')
     end
 
     def teardown
-      FileUtils.rm_rf TMPROOT if File.directory?(TMPROOT)
+      FileUtils.remove_entry_secure @tmproot if File.directory?(@tmproot)
+    end
+
+    def test_reader_open_notexist
+      assert_raise(Errno::ENOENT) {
+        GDBM.open("#{@tmproot}/a", 0666, GDBM::READER)
+      }
+    end
+
+    def test_writer_open_notexist
+      skip "gdbm_open(GDBM_WRITER) is broken on libgdbm 1.8.0" if /1\.8\.0/ =~ GDBM::VERSION
+
+      assert_raise(Errno::ENOENT) {
+        GDBM.open("#{@tmproot}/a", 0666, GDBM::WRITER)
+      }
+    end
+
+    def test_wrcreat_open_notexist
+      v = GDBM.open("#{@tmproot}/a", 0666, GDBM::WRCREAT)
+      assert_instance_of(GDBM, v)
+      v.close
+    end
+
+    def test_newdb_open_notexist
+      v = GDBM.open("#{@tmproot}/a", 0666, GDBM::NEWDB)
+      assert_instance_of(GDBM, v)
+      v.close
     end
 
     def test_reader_open
-      GDBM.open("#{TMPROOT}/a.dbm") {}
-      v = GDBM.open("#{TMPROOT}/a.dbm", nil, GDBM::READER) {|d|
-        assert_raises(GDBMError) { d["k"] = "v" }
+      GDBM.open("#{@tmproot}/a.dbm") {} # create a db.
+      v = GDBM.open("#{@tmproot}/a.dbm", nil, GDBM::READER) {|d|
+        assert_raise(GDBMError) { d["k"] = "v" }
         true
       }
       assert(v)
     end
 
     def test_newdb_open
-      GDBM.open("#{TMPROOT}/a.dbm") {|dbm|
+      GDBM.open("#{@tmproot}/a.dbm") {|dbm|
         dbm["k"] = "v"
-      } 
-      v = GDBM.open("#{TMPROOT}/a.dbm", nil, GDBM::NEWDB) {|d|
+      }
+      v = GDBM.open("#{@tmproot}/a.dbm", nil, GDBM::NEWDB) {|d|
         assert_equal(0, d.length)
         assert_nil(d["k"])
         true
@@ -683,9 +727,9 @@ if defined? GDBM
     end
 
     def test_freeze
-      GDBM.open("#{TMPROOT}/a.dbm") {|d|
+      GDBM.open("#{@tmproot}/a.dbm") {|d|
         d.freeze
-        assert_raises(TypeError) { d["k"] = "v" }
+        assert_raise(RuntimeError) { d["k"] = "v" }
       }
     end
   end
