@@ -1,8 +1,11 @@
+# -*- coding: us-ascii -*-
 require "test/unit"
 require "fileutils"
 require "tmpdir"
 
 class TestFileExhaustive < Test::Unit::TestCase
+  DRIVE = Dir.pwd[%r'\A(?:[a-z]:|//[^/]+/[^/]+)'i]
+
   def assert_incompatible_encoding
     d = "\u{3042}\u{3044}".encode("utf-16le")
     assert_raise(Encoding::CompatibilityError) {yield d}
@@ -12,6 +15,7 @@ class TestFileExhaustive < Test::Unit::TestCase
 
   def setup
     @dir = Dir.mktmpdir("rubytest-file")
+    @rootdir = "#{DRIVE}/"
     File.chown(-1, Process.gid, @dir)
     @file = make_tmp_filename("file")
     @zerofile = make_tmp_filename("zerofile")
@@ -83,7 +87,9 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_integer_or_nil(fs1.rdev_minor)
       assert_integer(fs1.ino)
       assert_integer(fs1.mode)
-      unless /emx/ =~ RUBY_PLATFORM
+      unless /emx|mswin|mingw/ =~ RUBY_PLATFORM
+        # on Windows, nlink is always 1. but this behavior will be changed
+        # in the future.
         assert_equal(@hardlinkfile ? 2 : 1, fs1.nlink)
       end
       assert_integer(fs1.uid)
@@ -352,11 +358,14 @@ class TestFileExhaustive < Test::Unit::TestCase
     assert_raise(Errno::EEXIST) { File.link(@file, @file) }
   end
 
-  def test_symlink2
+  def test_readlink
     return unless @symlinkfile
     assert_equal(@file, File.readlink(@symlinkfile))
     assert_raise(Errno::EINVAL) { File.readlink(@file) }
     assert_raise(Errno::ENOENT) { File.readlink(@nofile) }
+    if fs = Encoding.find("filesystem")
+      assert_equal(fs, File.readlink(@symlinkfile).encoding)
+    end
   rescue NotImplementedError
   end
 
@@ -391,12 +400,232 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_equal(@file, File.expand_path(@file + " "))
       assert_equal(@file, File.expand_path(@file + "."))
       assert_equal(@file, File.expand_path(@file + "::$DATA"))
+      assert_match(/\Ac:\//i, File.expand_path('c:'), '[ruby-core:31591]')
+      assert_match(/\Ac:\//i, File.expand_path('c:foo', 'd:/bar'))
+      assert_match(%r'\Ac:/bar/foo\z'i, File.expand_path('c:foo', 'c:/bar'))
     end
-    assert_kind_of(String, File.expand_path("~"))
+    if DRIVE
+      assert_match(%r"\Az:/foo\z"i, File.expand_path('/foo', "z:/bar"))
+      assert_match(%r"\A//host/share/foo\z"i, File.expand_path('/foo', "//host/share/bar"))
+      assert_match(%r"\A#{DRIVE}/foo\z"i, File.expand_path('/foo'))
+    else
+      assert_equal("/foo", File.expand_path('/foo'))
+    end
+  end
+
+  def test_expand_path_home
+    assert_kind_of(String, File.expand_path("~")) if ENV["HOME"]
     assert_raise(ArgumentError) { File.expand_path("~foo_bar_baz_unknown_user_wahaha") }
     assert_raise(ArgumentError) { File.expand_path("~foo_bar_baz_unknown_user_wahaha", "/") }
+    begin
+      bug3630 = '[ruby-core:31537]'
+      home = ENV["HOME"]
+      home_drive = ENV["HOMEDRIVE"]
+      home_path = ENV["HOMEPATH"]
+      user_profile = ENV["USERPROFILE"]
+      ENV["HOME"] = nil
+      ENV["HOMEDRIVE"] = nil
+      ENV["HOMEPATH"] = nil
+      ENV["USERPROFILE"] = nil
+      assert_raise(ArgumentError) { File.expand_path("~") }
+      ENV["HOME"] = "~"
+      assert_raise(ArgumentError, bug3630) { File.expand_path("~") }
+      ENV["HOME"] = "."
+      assert_raise(ArgumentError, bug3630) { File.expand_path("~") }
+    ensure
+      ENV["HOME"] = home
+      ENV["HOMEDRIVE"] = home_drive
+      ENV["HOMEPATH"] = home_path
+      ENV["USERPROFILE"] = user_profile
+    end
+  end
 
-    assert_incompatible_encoding {|d| File.expand_path(d)}
+  def test_expand_path_remove_trailing_alternative_data
+    assert_equal File.join(@rootdir, "aaa"), File.expand_path("#{@rootdir}/aaa::$DATA")
+    assert_equal File.join(@rootdir, "aa:a"), File.expand_path("#{@rootdir}/aa:a:$DATA")
+    assert_equal File.join(@rootdir, "aaa:$DATA"), File.expand_path("#{@rootdir}/aaa:$DATA")
+  end if DRIVE
+
+  def test_expand_path_resolve_empty_string_current_directory
+    assert_equal(Dir.pwd, File.expand_path(""))
+  end
+
+  def test_expand_path_resolve_dot_current_directory
+    assert_equal(Dir.pwd, File.expand_path("."))
+  end
+
+  def test_expand_path_resolve_file_name_relative_current_directory
+    assert_equal(File.join(Dir.pwd, "foo"), File.expand_path("foo"))
+  end
+
+  def test_ignore_nil_dir_string
+    assert_equal(File.join(Dir.pwd, "foo"), File.expand_path("foo", nil))
+  end
+
+  def test_expand_path_resolve_file_name_and_dir_string_relative
+    assert_equal(File.join(Dir.pwd, "bar", "foo"),
+      File.expand_path("foo", "bar"))
+  end
+
+  def test_expand_path_cleanup_dots_file_name
+    bug = "[ruby-talk:18512]"
+
+    assert_equal(File.join(Dir.pwd, ".a"), File.expand_path(".a"), bug)
+    assert_equal(File.join(Dir.pwd, "..a"), File.expand_path("..a"), bug)
+
+    if DRIVE
+      # cleanup dots only on Windows
+      assert_equal(File.join(Dir.pwd, "a"), File.expand_path("a."), bug)
+      skip "FIXME"
+      assert_equal(File.join(Dir.pwd, "a"), File.expand_path("a.."), bug)
+    else
+      assert_equal(File.join(Dir.pwd, "a."), File.expand_path("a."), bug)
+      assert_equal(File.join(Dir.pwd, "a.."), File.expand_path("a.."), bug)
+    end
+  end
+
+  def test_expand_path_converts_a_pathname_to_an_absolute_pathname_using_a_complete_path
+    assert_equal(@dir, File.expand_path("", "#{@dir}"))
+    assert_equal(File.join(@dir, "a"), File.expand_path("a", "#{@dir}"))
+    assert_equal(File.join(@dir, "a"), File.expand_path("../a", "#{@dir}/xxx"))
+    assert_equal(@rootdir, File.expand_path(".", "#{@rootdir}"))
+  end
+
+  def test_expand_path_ignores_supplied_dir_if_path_contains_a_drive_letter
+    assert_equal(@rootdir, File.expand_path(@rootdir, "D:/"))
+  end if DRIVE
+
+  def test_expand_path_removes_trailing_slashes_from_absolute_path
+    assert_equal(File.join(@rootdir, "foo"), File.expand_path("#{@rootdir}foo/"))
+    assert_equal(File.join(@rootdir, "foo.rb"), File.expand_path("#{@rootdir}foo.rb/"))
+  end
+
+  def test_expand_path_removes_trailing_spaces_from_absolute_path
+    assert_equal(File.join(@rootdir, "a"), File.expand_path("#{@rootdir}a "))
+  end if DRIVE
+
+  def test_expand_path_converts_a_pathname_which_starts_with_a_slash_using_dir_s_drive
+    assert_match(%r"\Az:/foo\z"i, File.expand_path('/foo', "z:/bar"))
+  end if DRIVE
+
+  def test_expand_path_converts_a_pathname_which_starts_with_a_slash_and_unc_pathname
+    assert_equal("//foo", File.expand_path('//foo', "//bar"))
+    assert_equal("//bar/foo", File.expand_path('/foo', "//bar"))
+    assert_equal("//foo", File.expand_path('//foo', "/bar"))
+  end if DRIVE
+
+  def test_expand_path_converts_a_dot_with_unc_dir
+    assert_equal("//", File.expand_path('.', "//"))
+  end
+
+  def test_expand_path_preserves_unc_path_root
+    assert_equal("//", File.expand_path("//"))
+    assert_equal("//", File.expand_path("//."))
+    assert_equal("//", File.expand_path("//.."))
+  end
+
+  def test_expand_path_converts_a_pathname_which_starts_with_a_slash_using_host_share
+    assert_match(%r"\A//host/share/foo\z"i, File.expand_path('/foo', "//host/share/bar"))
+  end if DRIVE
+
+  def test_expand_path_converts_a_pathname_which_starts_with_a_slash_using_a_current_drive
+    assert_match(%r"\A#{DRIVE}/foo\z"i, File.expand_path('/foo'))
+  end
+
+  def test_expand_path_returns_tainted_strings_or_not
+    assert_equal(true, File.expand_path('foo').tainted?)
+    assert_equal(true, File.expand_path('foo'.taint).tainted?)
+    assert_equal(true, File.expand_path('/foo'.taint).tainted?)
+    assert_equal(true, File.expand_path('foo', 'bar').tainted?)
+    assert_equal(true, File.expand_path('foo', '/bar'.taint).tainted?)
+    assert_equal(true, File.expand_path('foo'.taint, '/bar').tainted?)
+    assert_equal(true, File.expand_path('~').tainted?) if ENV["HOME"]
+
+    if DRIVE
+      assert_equal(true, File.expand_path('/foo').tainted?)
+      assert_equal(false, File.expand_path('//foo').tainted?)
+      assert_equal(true, File.expand_path('C:/foo'.taint).tainted?)
+      assert_equal(false, File.expand_path('C:/foo').tainted?)
+      assert_equal(true, File.expand_path('foo', '/bar').tainted?)
+      assert_equal(true, File.expand_path('foo', 'C:/bar'.taint).tainted?)
+      assert_equal(true, File.expand_path('foo'.taint, 'C:/bar').tainted?)
+      assert_equal(false, File.expand_path('foo', 'C:/bar').tainted?)
+      assert_equal(false, File.expand_path('C:/foo/../bar').tainted?)
+      assert_equal(false, File.expand_path('foo', '//bar').tainted?)
+    else
+      assert_equal(false, File.expand_path('/foo').tainted?)
+      assert_equal(false, File.expand_path('foo', '/bar').tainted?)
+    end
+  end
+
+  def test_expand_path_converts_a_pathname_to_an_absolute_pathname_using_home_as_base
+    old_home = ENV["HOME"]
+    home = ENV["HOME"] = "#{DRIVE}/UserHome"
+    assert_equal(home, File.expand_path("~"))
+    assert_equal(home, File.expand_path("~", "C:/FooBar"))
+    assert_equal(File.join(home, "a"), File.expand_path("~/a", "C:/FooBar"))
+  ensure
+    ENV["HOME"] = old_home
+  end
+
+  def test_expand_path_converts_a_pathname_to_an_absolute_pathname_using_unc_home
+    old_home = ENV["HOME"]
+    unc_home = ENV["HOME"] = "//UserHome"
+    assert_equal(unc_home, File.expand_path("~"))
+  ensure
+    ENV["HOME"] = old_home
+  end if DRIVE
+
+  def test_expand_path_does_not_modify_a_home_string_argument
+    old_home = ENV["HOME"]
+    home = ENV["HOME"] = "#{DRIVE}/UserHome"
+    str = "~/a"
+    assert_equal("#{home}/a", File.expand_path(str))
+    assert_equal("~/a", str)
+  ensure
+    ENV["HOME"] = old_home
+  end
+
+  def test_expand_path_raises_argument_error_for_any_supplied_username
+    bug = '[ruby-core:39597]'
+    assert_raise(ArgumentError, bug) { File.expand_path("~anything") }
+  end if DRIVE
+
+  def test_expand_path_raises_a_type_error_if_not_passed_a_string_type
+    assert_raise(TypeError) { File.expand_path(1) }
+    assert_raise(TypeError) { File.expand_path(nil) }
+    assert_raise(TypeError) { File.expand_path(true) }
+  end
+
+  def test_expand_path_expands_dot_dir
+    assert_equal("#{DRIVE}/dir", File.expand_path("#{DRIVE}/./dir"))
+  end
+
+  def test_expand_path_does_not_expand_wildcards
+    assert_equal("#{DRIVE}/*", File.expand_path("./*", "#{DRIVE}/"))
+    assert_equal("#{Dir.pwd}/*", File.expand_path("./*", Dir.pwd))
+    assert_equal("#{DRIVE}/?", File.expand_path("./?", "#{DRIVE}/"))
+    assert_equal("#{Dir.pwd}/?", File.expand_path("./?", Dir.pwd))
+  end if DRIVE
+
+  def test_expand_path_does_not_modify_the_string_argument
+    str = "./a/b/../c"
+    assert_equal("#{Dir.pwd}/a/c", File.expand_path(str, Dir.pwd))
+    assert_equal("./a/b/../c", str)
+  end
+
+  def test_expand_path_returns_a_string_when_passed_a_string_subclass
+    sub = Class.new(String)
+    str = sub.new "./a/b/../c"
+    path = File.expand_path(str, Dir.pwd)
+    assert_equal("#{Dir.pwd}/a/c", path)
+    assert_instance_of(String, path)
+  end
+
+  def test_expand_path_accepts_objects_that_have_a_to_path_method
+    klass = Class.new { def to_path; "a/b/c"; end }
+    obj = klass.new
+    assert_equal("#{Dir.pwd}/a/b/c", File.expand_path(obj))
   end
 
   def test_basename
@@ -421,18 +650,35 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_equal(basename, File.basename(@file + ".", ".*"))
       assert_equal(basename, File.basename(@file + "::$DATA", ".*"))
     end
+    if File::ALT_SEPARATOR == '\\'
+      a = "foo/\225\\\\"
+      [%W"cp437 \225", %W"cp932 \225\\"].each do |cp, expected|
+        assert_equal(expected.force_encoding(cp), File.basename(a.dup.force_encoding(cp)), cp)
+      end
+    end
 
     assert_incompatible_encoding {|d| File.basename(d)}
+    assert_incompatible_encoding {|d| File.basename(d, ".*")}
+    assert_raise(Encoding::CompatibilityError) {File.basename("foo.ext", ".*".encode("utf-16le"))}
+
+    s = "foo\x93_a".force_encoding("cp932")
+    assert_equal(s, File.basename(s, "_a"))
   end
 
   def test_dirname
     assert(@file.start_with?(File.dirname(@file)))
     assert_equal(".", File.dirname(""))
     assert_incompatible_encoding {|d| File.dirname(d)}
+    if File::ALT_SEPARATOR == '\\'
+      a = "\225\\\\foo"
+      [%W"cp437 \225", %W"cp932 \225\\"].each do |cp, expected|
+        assert_equal(expected.force_encoding(cp), File.dirname(a.dup.force_encoding(cp)), cp)
+      end
+    end
   end
 
   def test_extname
-    assert(".test", File.extname(@file))
+    assert_equal(".test", File.extname(@file))
     prefixes = ["", "/", ".", "/.", "bar/.", "/bar/."]
     infixes = ["", " ", "."]
     infixes2 = infixes + [".ext "]
@@ -452,6 +698,8 @@ class TestFileExhaustive < Test::Unit::TestCase
         end
       end
     end
+    bug3175 = '[ruby-core:29627]'
+    assert_equal(".rb", File.extname("/tmp//bla.rb"), bug3175)
 
     assert_incompatible_encoding {|d| File.extname(d)}
   end
@@ -470,6 +718,13 @@ class TestFileExhaustive < Test::Unit::TestCase
     def o.to_path; "foo"; end
     assert_equal(s, File.join(o, "bar", "baz"))
     assert_equal(s, File.join("foo" + File::SEPARATOR, "bar", File::SEPARATOR + "baz"))
+    if File::ALT_SEPARATOR == '\\'
+      a = "\225\\"
+      b = "foo"
+      [%W"cp437 \225\\foo", %W"cp932 \225\\/foo"].each do |cp, expected|
+        assert_equal(expected.force_encoding(cp), File.join(a.dup.force_encoding(cp), b.dup.force_encoding(cp)), cp)
+      end
+    end
   end
 
   def test_truncate
@@ -492,7 +747,7 @@ class TestFileExhaustive < Test::Unit::TestCase
     f.close
     make_file("foo", @file)
 
-    assert_raise(IOError) { File.open(@file) {|f| f.truncate(0)} }
+    assert_raise(IOError) { File.open(@file) {|ff| ff.truncate(0)} }
   rescue NotImplementedError
   end
 
@@ -530,7 +785,7 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_equal(File.socket?(f), test(?S, f))
       assert_equal(File.setuid?(f), test(?u, f))
       assert_equal(File.writable?(f), test(?w, f))
-      assert_equal(File.world_writable?(f), test(?W, f))
+      assert_equal(File.writable_real?(f), test(?W, f))
       assert_equal(File.executable?(f), test(?x, f))
       assert_equal(File.executable_real?(f), test(?X, f))
       assert_equal(File.zero?(f), test(?z, f))
@@ -570,7 +825,9 @@ class TestFileExhaustive < Test::Unit::TestCase
       assert_integer_or_nil(fs1.rdev_minor)
       assert_integer(fs1.ino)
       assert_integer(fs1.mode)
-      unless /emx/ =~ RUBY_PLATFORM
+      unless /emx|mswin|mingw/ =~ RUBY_PLATFORM
+        # on Windows, nlink is always 1. but this behavior will be changed
+        # in the future.
         assert_equal(@hardlinkfile ? 2 : 1, fs1.nlink)
       end
       assert_integer(fs1.uid)
@@ -738,6 +995,13 @@ class TestFileExhaustive < Test::Unit::TestCase
     assert_equal(0, File::Stat.new(@zerofile).size)
   end
 
+  def test_stat_special_file
+    # test for special files such as pagefile.sys on Windows
+    assert_nothing_raised do
+      Dir::glob("C:/*.sys") {|f| File::Stat.new(f) }
+    end
+  end if DRIVE
+
   def test_path_check
     assert_nothing_raised { ENV["PATH"] }
   end
