@@ -19,6 +19,10 @@ class TestTime < Test::Unit::TestCase
     assert_equal(Time.utc(2000,2,10), Time.new(2000,2,9, 13,0,0, -3600*11))
     assert_equal(Time.utc(2000,2,10), Time.new(2000,2,10, 11,0,0, "+11:00"))
     assert_equal(Rational(1,2), Time.new(2000,2,10, 11,0,5.5, "+11:00").subsec)
+    bug4090 = '[ruby-dev:42631]'
+    tm = [2001,2,28,23,59,30]
+    t = Time.new(*tm, "-12:00")
+    assert_equal([2001,2,28,23,59,30,-43200], [t.year, t.month, t.mday, t.hour, t.min, t.sec, t.gmt_offset], bug4090)
   end
 
   def test_time_add()
@@ -27,6 +31,16 @@ class TestTime < Test::Unit::TestCase
     assert_equal(Time.utc(2000, 3, 21, 3, 30) + (-3 * 3600),
                  Time.utc(2000, 3, 21, 0, 30))
     assert_equal(0, (Time.at(1.1) + 0.9).usec)
+
+    assert((Time.utc(2000, 4, 1) + 24).utc?)
+    assert(!(Time.local(2000, 4, 1) + 24).utc?)
+
+    t = Time.new(2000, 4, 1, 0, 0, 0, "+01:00") + 24
+    assert(!t.utc?)
+    assert_equal(3600, t.utc_offset)
+    t = Time.new(2000, 4, 1, 0, 0, 0, "+02:00") + 24
+    assert(!t.utc?)
+    assert_equal(7200, t.utc_offset)
   end
 
   def test_time_subt()
@@ -152,6 +166,45 @@ class TestTime < Test::Unit::TestCase
       assert_equal(999999998, Time.at(-14e-10).nsec)
       assert_equal(999999998, Time.at(-16e-10).nsec)
     end
+
+    t = Time.at(-4611686019).utc
+    assert_equal(1823, t.year)
+
+    t = Time.at(4611686018, 999999).utc
+    assert_equal(2116, t.year)
+    assert_equal("0.999999".to_r, t.subsec)
+
+    t = Time.at(2**40 + "1/3".to_r, 9999999999999).utc
+    assert_equal(36812, t.year)
+
+    t = Time.at(-0x3fff_ffff_ffff_ffff)
+    assert_equal(-146138510344, t.year)
+    t = Time.at(-0x4000_0000_0000_0000)
+    assert_equal(-146138510344, t.year)
+    t = Time.at(-0x4000_0000_0000_0001)
+    assert_equal(-146138510344, t.year)
+    t = Time.at(-0x5000_0000_0000_0001)
+    assert_equal(-182673138422, t.year)
+    t = Time.at(-0x6000_0000_0000_0000)
+    assert_equal(-219207766501, t.year)
+
+    t = Time.at(0).utc
+    assert_equal([1970,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400).utc
+    assert_equal([1969,12,31, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400 * (400 * 365 + 97)).utc
+    assert_equal([1970-400,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400 * (400 * 365 + 97)*1000).utc
+    assert_equal([1970-400*1000,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400 * (400 * 365 + 97)*2421).utc
+    assert_equal([1970-400*2421,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-86400 * (400 * 365 + 97)*1000000).utc
+    assert_equal([1970-400*1000000,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+
+    t = Time.at(-30613683110400).utc
+    assert_equal([-968139,1,1, 0,0,0], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
+    t = Time.at(-30613683110401).utc
+    assert_equal([-968140,12,31, 23,59,59], [t.year, t.mon, t.day, t.hour, t.min, t.sec])
   end
 
   def test_at2
@@ -191,6 +244,18 @@ class TestTime < Test::Unit::TestCase
     assert_marshal_roundtrip(Time.at(0, 0.120))
   end
 
+  def test_marshal_nsec_191
+    # generated by ruby 1.9.1p376
+    m = "\x04\bIu:\tTime\r \x80\x11\x80@\xE2\x01\x00\x06:\rsubmicro\"\ax\x90"
+    t = Marshal.load(m)
+    assert_equal(Time.at(Rational(123456789, 1000000000)), t, "[ruby-dev:40133]")
+  end
+
+  def test_marshal_rational
+    assert_marshal_roundtrip(Time.at(0, Rational(1,3)))
+    assert_not_match(/Rational/, Marshal.dump(Time.at(0, Rational(1,3))))
+  end
+
   def test_marshal_ivar
     t = Time.at(123456789, 987654.321)
     t.instance_eval { @var = 135 }
@@ -198,6 +263,27 @@ class TestTime < Test::Unit::TestCase
     assert_marshal_roundtrip(Marshal.load(Marshal.dump(t)))
   end
 
+  def test_marshal_timezone
+    bug = '[ruby-dev:40063]'
+
+    t1 = Time.gm(2000)
+    m = Marshal.dump(t1.getlocal("-02:00"))
+    t2 = Marshal.load(m)
+    assert_equal(t1, t2)
+    assert_equal(-7200, t2.utc_offset, bug)
+    m = Marshal.dump(t1.getlocal("+08:15"))
+    t2 = Marshal.load(m)
+    assert_equal(t1, t2)
+    assert_equal(29700, t2.utc_offset, bug)
+  end
+
+  def test_marshal_to_s
+    t1 = Time.new(2011,11,8, 0,42,25, 9*3600)
+    t2 = Time.at(Marshal.load(Marshal.dump(t1)))
+    assert_equal("2011-11-08 00:42:25 +0900", t2.to_s,
+      "[ruby-dev:44827] [Bug #5586]")
+  end
+
   # Sat Jan 01 00:00:00 UTC 2000
   T2000 = Time.at(946684800).gmtime
 
@@ -251,6 +337,9 @@ class TestTime < Test::Unit::TestCase
 
     t = Time.local(2000)
     assert_equal(t.gmt_offset, T2000 - t)
+
+    assert_equal(-4427700000, Time.utc(-4427700000,12,1).year)
+    assert_equal(-2**30+10, Time.utc(-2**30+10,1,1).year)
   end
 
   def test_time_interval
@@ -351,6 +440,31 @@ class TestTime < Test::Unit::TestCase
     assert_equal(T2000 + 1, T2000.succ)
   end
 
+  def test_plus_type
+    t0 = Time.utc(2000,1,1)
+    n0 = t0.to_i
+    n1 = n0+1
+    t1 = Time.at(n1)
+    assert_equal(t1, t0 + 1)
+    assert_equal(t1, t0 + 1.0)
+    assert_equal(t1, t0 + Rational(1,1))
+    assert_equal(t1, t0 + SimpleDelegator.new(1))
+    assert_equal(t1, t0 + SimpleDelegator.new(1.0))
+    assert_equal(t1, t0 + SimpleDelegator.new(Rational(1,1)))
+    assert_raise(TypeError) { t0 + nil }
+    assert_raise(TypeError) { t0 + "1" }
+    assert_raise(TypeError) { t0 + SimpleDelegator.new("1") }
+    assert_equal(0.5, (t0 + 1.5).subsec)
+    assert_equal(Rational(1,3), (t0 + Rational(4,3)).subsec)
+    assert_equal(0.5, (t0 + SimpleDelegator.new(1.5)).subsec)
+    assert_equal(Rational(1,3), (t0 + SimpleDelegator.new(Rational(4,3))).subsec)
+  end
+
+  def test_minus
+    t = Time.at(-4611686018).utc - 100
+    assert_equal(1823, t.year)
+  end
+
   def test_readers
     assert_equal(0, T2000.sec)
     assert_equal(0, T2000.min)
@@ -362,6 +476,7 @@ class TestTime < Test::Unit::TestCase
     assert_equal(1, T2000.yday)
     assert_equal(false, T2000.isdst)
     assert_equal("UTC", T2000.zone)
+    assert_equal(Encoding.find("locale"), T2000.zone.encoding)
     assert_equal(0, T2000.gmt_offset)
     assert(!T2000.sunday?)
     assert(!T2000.monday?)
@@ -383,6 +498,7 @@ class TestTime < Test::Unit::TestCase
     assert_equal(t.yday, Time.at(946684800).yday)
     assert_equal(t.isdst, Time.at(946684800).isdst)
     assert_equal(t.zone, Time.at(946684800).zone)
+    assert_equal(Encoding.find("locale"), Time.at(946684800).zone.encoding)
     assert_equal(t.gmt_offset, Time.at(946684800).gmt_offset)
     assert_equal(t.sunday?, Time.at(946684800).sunday?)
     assert_equal(t.monday?, Time.at(946684800).monday?)
@@ -481,6 +597,18 @@ class TestTime < Test::Unit::TestCase
     assert_equal("02", t.strftime("%0l"))
     assert_equal(" 2", t.strftime("%_l"))
 
+    t = Time.utc(1,1,4)
+    assert_equal("0001", t.strftime("%Y"))
+    assert_equal("0001", t.strftime("%G"))
+
+    t = Time.utc(0,1,4)
+    assert_equal("0000", t.strftime("%Y"))
+    assert_equal("0000", t.strftime("%G"))
+
+    t = Time.utc(-1,1,4)
+    assert_equal("-0001", t.strftime("%Y"))
+    assert_equal("-0001", t.strftime("%G"))
+
     # [ruby-dev:37155]
     t = Time.mktime(1970, 1, 18)
     assert_equal("0", t.strftime("%w"))
@@ -505,6 +633,64 @@ class TestTime < Test::Unit::TestCase
     assert_equal("JANUARY", T2000.strftime("%#B"))
     assert_equal("JAN", T2000.strftime("%#h"))
     assert_equal("FRIDAY", Time.local(2008,1,4).strftime("%#A"))
+
+    t = Time.utc(2000,3,14, 6,53,"58.979323846".to_r) # Pi Day
+    assert_equal("03/14/2000  6:53:58.97932384600000000000000000000",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%29N"))
+    assert_equal("03/14/2000  6:53:58.9793238460",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%10N"))
+    assert_equal("03/14/2000  6:53:58.979323846",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%9N"))
+    assert_equal("03/14/2000  6:53:58.97932384",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%8N"))
+
+    t = Time.utc(1592,3,14, 6,53,"58.97932384626433832795028841971".to_r) # Pi Day
+    assert_equal("03/14/1592  6:53:58.97932384626433832795028841971",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%29N"))
+    assert_equal("03/14/1592  6:53:58.9793238462",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%10N"))
+    assert_equal("03/14/1592  6:53:58.979323846",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%9N"))
+    assert_equal("03/14/1592  6:53:58.97932384",
+                 t.strftime("%m/%d/%Y %l:%M:%S.%8N"))
+
+    # [ruby-core:33985]
+    assert_equal("3000000000", Time.at(3000000000).strftime('%s'))
+
+    bug4457 = '[ruby-dev:43285]'
+    assert_raise(Errno::ERANGE, bug4457) {Time.now.strftime('%8192z')}
+
+    bug4458 = '[ruby-dev:43287]'
+    t = T2000.getlocal("+09:00")
+    assert_equal("      +900", t.strftime("%_10z"), bug4458)
+    assert_equal("+000000900", t.strftime("%10z"), bug4458)
+    assert_equal("     +9:00", t.strftime("%_:10z"), bug4458)
+    assert_equal("+000009:00", t.strftime("%:10z"), bug4458)
+    assert_equal("  +9:00:00", t.strftime("%_::10z"), bug4458)
+    assert_equal("+009:00:00", t.strftime("%::10z"), bug4458)
+    t = T2000.getlocal("-05:00")
+    assert_equal("      -500", t.strftime("%_10z"), bug4458)
+    assert_equal("-000000500", t.strftime("%10z"), bug4458)
+    assert_equal("     -5:00", t.strftime("%_:10z"), bug4458)
+    assert_equal("-000005:00", t.strftime("%:10z"), bug4458)
+    assert_equal("  -5:00:00", t.strftime("%_::10z"), bug4458)
+    assert_equal("-005:00:00", t.strftime("%::10z"), bug4458)
+
+    bug6323 = '[ruby-core:44447]'
+    t = T2000.getlocal("+00:36")
+    assert_equal("      +036", t.strftime("%_10z"), bug6323)
+    assert_equal("+000000036", t.strftime("%10z"), bug6323)
+    assert_equal("     +0:36", t.strftime("%_:10z"), bug6323)
+    assert_equal("+000000:36", t.strftime("%:10z"), bug6323)
+    assert_equal("  +0:36:00", t.strftime("%_::10z"), bug6323)
+    assert_equal("+000:36:00", t.strftime("%::10z"), bug6323)
+    t = T2000.getlocal("-00:55")
+    assert_equal("      -055", t.strftime("%_10z"), bug6323)
+    assert_equal("-000000055", t.strftime("%10z"), bug6323)
+    assert_equal("     -0:55", t.strftime("%_:10z"), bug6323)
+    assert_equal("-000000:55", t.strftime("%:10z"), bug6323)
+    assert_equal("  -0:55:00", t.strftime("%_::10z"), bug6323)
+    assert_equal("-000:55:00", t.strftime("%::10z"), bug6323)
   end
 
   def test_delegate
@@ -515,4 +701,56 @@ class TestTime < Test::Unit::TestCase
     assert_equal(-1, d1 <=> d2)
     assert_equal(1, d2 <=> d1)
   end
+
+  def test_to_r
+    assert_kind_of(Rational, Time.new(2000,1,1,0,0,Rational(4,3)).to_r)
+    assert_kind_of(Rational, Time.utc(1970).to_r)
+  end
+
+  def test_round
+    t = Time.utc(1999,12,31, 23,59,59)
+    t2 = (t+0.4).round
+    assert_equal([59,59,23, 31,12,1999, 5,365,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+0.49).round
+    assert_equal([59,59,23, 31,12,1999, 5,365,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+0.5).round
+    assert_equal([0,0,0, 1,1,2000, 6,1,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+1.4).round
+    assert_equal([0,0,0, 1,1,2000, 6,1,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+1.49).round
+    assert_equal([0,0,0, 1,1,2000, 6,1,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+    t2 = (t+1.5).round
+    assert_equal([1,0,0, 1,1,2000, 6,1,false,"UTC"], t2.to_a)
+    assert_equal(0, t2.subsec)
+
+    t2 = (t+0.123456789).round(4)
+    assert_equal([59,59,23, 31,12,1999, 5,365,false,"UTC"], t2.to_a)
+    assert_equal(Rational(1235,10000), t2.subsec)
+
+    off = 0.0
+    100.times {|i|
+      t2 = (t+off).round(1)
+      assert_equal(Rational(i % 10, 10), t2.subsec)
+      off += 0.1
+    }
+  end
+
+  def test_getlocal_dont_share_eigenclass
+    bug5012 = "[ruby-dev:44071]"
+
+    t0 = Time.now
+    class <<t0; end
+    t1 = t0.getlocal
+
+    def t0.m
+      0
+    end
+
+    assert_raise(NoMethodError, bug5012) { t1.m }
+  end
 end
