@@ -1,13 +1,6 @@
-#--
-# Copyright 2006 by Chad Fowler, Rich Kilmer, Jim Weirich and others.
-# All rights reserved.
-# See LICENSE.txt for permissions.
-#++
+require 'rubygems/test_case'
 
-require_relative 'gemutilities'
-require 'rubygems/package'
-
-class TestKernel < RubyGemTestCase
+class TestKernel < Gem::TestCase
 
   def setup
     super
@@ -26,21 +19,18 @@ class TestKernel < RubyGemTestCase
   def test_gem
     assert gem('a', '= 1'), "Should load"
     assert $:.any? { |p| %r{a-1/lib} =~ p }
-    assert $:.any? { |p| %r{a-1/bin} =~ p }
   end
 
-  def test_gem_redundent
+  def test_gem_redundant
     assert gem('a', '= 1'), "Should load"
     refute gem('a', '= 1'), "Should not load"
     assert_equal 1, $:.select { |p| %r{a-1/lib} =~ p }.size
-    assert_equal 1, $:.select { |p| %r{a-1/bin} =~ p }.size
   end
 
   def test_gem_overlapping
     assert gem('a', '= 1'), "Should load"
     refute gem('a', '>= 1'), "Should not load"
     assert_equal 1, $:.select { |p| %r{a-1/lib} =~ p }.size
-    assert_equal 1, $:.select { |p| %r{a-1/bin} =~ p }.size
   end
 
   def test_gem_conflicting
@@ -50,16 +40,16 @@ class TestKernel < RubyGemTestCase
       gem 'a', '= 2'
     end
 
-    assert_match(/activate a \(= 2, runtime\)/, ex.message)
+    assert_equal "can't activate a-2, already activated a-1", ex.message
     assert_match(/activated a-1/, ex.message)
     assert_equal 'a', ex.name
-    assert_equal Gem::Requirement.new('= 2'), ex.version_requirement
 
     assert $:.any? { |p| %r{a-1/lib} =~ p }
-    assert $:.any? { |p| %r{a-1/bin} =~ p }
     refute $:.any? { |p| %r{a-2/lib} =~ p }
-    refute $:.any? { |p| %r{a-2/bin} =~ p }
   end
 
+  def test_gem_not_adding_bin
+    assert gem('a', '= 1'), "Should load"
+    refute $:.any? { |p| %r{a-1/bin} =~ p }
+  end
 end
-
