@@ -5,7 +5,7 @@ require 'rbconfig'
 class TestBackquote < Test::Unit::TestCase
   include TestHelper
 
-  WINDOWS = Config::CONFIG['host_os'] =~ /Windows|mswin/
+  WINDOWS = RbConfig::CONFIG['host_os'] =~ /Windows|mswin/
   def test_backquote_special_commands
     if File.exists?("/bin/echo")
       output = `/bin/echo hello`
@@ -13,6 +13,21 @@ class TestBackquote < Test::Unit::TestCase
     end
   end
 
+  def test_backquote_special_commands_and_cwd_inside_classloader
+    # not sure why it fails with java-1.6 - assume it is rare feature
+    # and works for java-1.7+
+    if File.exists?("/bin/echo") and not ENV_JAVA['java.version'].start_with?("1.6.")
+      begin
+        cwd = Dir.pwd
+        Dir.chdir('uri:classloader:/')
+        output = `/bin/echo hello`
+        assert_equal("hello\n", output)
+      ensure
+        Dir.chdir(cwd)
+      end
+    end
+  end
+
   def test_system_special_commands
     if File.exists?("/bin/true")
       assert(system("/bin/true"))
@@ -25,16 +40,24 @@ class TestBackquote < Test::Unit::TestCase
     end
   end
 
-  def test_backquote_ruby
-    assert_equal "true\n", `ruby -e "puts true"`
-  end
-
   #JRUBY-2251
   def test_empty_backquotes
     if (!WINDOWS)
-      assert_equal("", ``)    # empty
-      assert_equal("", `   `) # spaces
-      assert_equal("", `\n`)
+      if RUBY_VERSION > '1.9' && ENV_JAVA['java.specification.version'] > '1.6'
+        # works as expected on Java 7+ (using process launching API)
+        assert_raise(Errno::ENOENT) {``}
+        assert_raise(Errno::ENOENT) {`   `}
+        #assert_raise(Errno::ENOENT) {`\n`}
+        pend "#{__method__}: `\\n` does not raise Errno::ENOENT as expected"
+      else
+        if RUBY_VERSION < '1.9' && ENV_JAVA['java.specification.version'] > '1.6'
+          pend "#{__method__}: `` does not work the same as MRI 1.8.7 (under Java 7+)"
+        else
+          assert_equal("", ``)    # empty
+          assert_equal("", `   `) # spaces
+          assert_equal("", `\n`)
+        end
+      end
     else # we just check that empty backquotes won't blow up JRuby
       ``    rescue nil
       `   ` rescue nil
@@ -53,10 +76,15 @@ class TestBackquote < Test::Unit::TestCase
 
       assert_equal "arguments: one two\n", `./arguments one two 2> /dev/null`
       assert_equal "", `./arguments three four > /dev/null`
-      ruby = File.join(Config::CONFIG['bindir'], Config::CONFIG['ruby_install_name'])
+      # ruby = File.join(RbConfig::CONFIG['bindir'], RbConfig::CONFIG['ruby_install_name'])
       assert_equal "arguments: five six\n", jruby(%{-e 'puts "arguments: five six"' 2> /dev/null})
     end
   ensure
     File.delete("arguments") rescue nil
   end
+
+  private
+
+  def pend(msg); warn msg end unless method_defined? :pend
+
 end
