@@ -5,7 +5,11 @@ class TestTimeNilOps < Test::Unit::TestCase
       Time.now - ()
     rescue TypeError=>x
       assert x
-      assert_equal "no implicit conversion to float from nil", x.message
+      if RUBY_VERSION =~ /1\.9/
+        assert_equal "no implicit conversion to rational from nil", x.message
+      else
+        assert_equal "no implicit conversion to float from nil", x.message
+      end
     end
   end
   def test_plus
@@ -13,7 +17,11 @@ class TestTimeNilOps < Test::Unit::TestCase
       Time.now + ()
     rescue TypeError=>x
       assert x
-      assert_equal "no implicit conversion to float from nil", x.message
+      if RUBY_VERSION =~ /1\.9/
+        assert_equal "no implicit conversion to rational from nil", x.message
+      else
+        assert_equal "no implicit conversion to float from nil", x.message
+      end
     end
   end
   def test_times
@@ -29,7 +37,7 @@ class TestTimeNilOps < Test::Unit::TestCase
   def test_div
     t = Time.now
     begin
-      Time.now / ()
+      t / ()
       fail "bleh"
     rescue NoMethodError=>x
       assert x
